@@ -6,6 +6,7 @@ from shutil import rmtree
 class TreesappTester(unittest.TestCase):
     def setUp(self) -> None:
         from .testing_utils import get_test_data
+        self.num_procs = 4
         # FASTA files
         self.aa_test_fa = get_test_data("marker_test_suite.faa")
         self.nt_test_fa = get_test_data("marker_test_suite.fna")
@@ -30,6 +31,23 @@ class TreesappTester(unittest.TestCase):
                 rmtree(output_dir)
         return
 
+    def test_abundance(self):
+        from treesapp.commands import abundance
+        from treesapp.file_parsers import read_classification_table
+        from .testing_utils import get_test_data
+        classification_table = os.path.join(self.ts_assign_output, "final_outputs", "marker_contig_map.tsv")
+        pre_lines = read_classification_table(get_test_data(classification_table))
+        abundance_command_list = ["--treesapp_output", self.ts_assign_output,
+                                  "--reads", get_test_data("test_TarA.1.fq"),
+                                  "--reverse", get_test_data("test_TarA.2.fq"),
+                                  "--pairing", "pe",
+                                  "--num_procs", str(self.num_procs),
+                                  "--delete"]
+        abundance(abundance_command_list)
+        post_lines = read_classification_table(get_test_data(classification_table))
+        self.assertEqual(len(pre_lines), len(post_lines))
+        return
+
     def test_assign_prot(self):
         ref_pkgs = ["McrA", "M0702", "S0001"]
         from treesapp.commands import assign
@@ -37,10 +55,11 @@ class TreesappTester(unittest.TestCase):
         assign_commands_list = ["--fastx_input", self.aa_test_fa,
                                 "--targets", ','.join(ref_pkgs),
                                 "--refpkg_dir", self.refpkg_dir,
-                                "--num_procs", str(2),
+                                "--num_procs", str(self.num_procs),
                                 "-m", "prot",
                                 "--output", "./TreeSAPP_assign/",
                                 "--stringency", "relaxed",
+                                "--placement_summary", "max_lwr",
                                 "--trim_align", "--overwrite", "--delete", "--svm"]
         assign(assign_commands_list)
         lines = read_classification_table("./TreeSAPP_assign/final_outputs/marker_contig_map.tsv")
@@ -53,7 +72,7 @@ class TreesappTester(unittest.TestCase):
         from treesapp.file_parsers import read_classification_table
         assign_commands_list = ["--fastx_input", self.nt_test_fa,
                                 "--targets", ','.join(ref_pkgs),
-                                "--num_procs", str(2),
+                                "--num_procs", str(self.num_procs),
                                 "-m", "dna",
                                 "--output", "./TreeSAPP_assign/",
                                 "--stringency", "strict",
@@ -89,6 +108,7 @@ class TreesappTester(unittest.TestCase):
         from .testing_utils import get_test_data
         create_commands_list = ["--fastx_input", get_test_data("create_test.faa"),
                                 "--accession2taxid", get_test_data("create_test.accession2taxid"),
+                                "--seqs2lineage", get_test_data("create_test_accession2lin.tsv"),
                                 "--refpkg_name", "Crt",
                                 "--similarity", "0.95",
                                 "--bootstraps", str(0),
@@ -96,13 +116,13 @@ class TreesappTester(unittest.TestCase):
                                 "--screen", "Bacteria", "--filter", "Archaea",
                                 "--min_taxonomic_rank", 'p',
                                 "--output", "./TreeSAPP_create",
-                                "--num_proc", str(2),
+                                "--num_procs", str(self.num_procs),
                                 "--trim_align", "--cluster", "--fast", "--headless", "--overwrite", "--delete"]
         create(create_commands_list)
         test_refpkg = ReferencePackage()
         test_refpkg.f__json = "./TreeSAPP_create/final_outputs/Crt_build.pkl"
         test_refpkg.slurp()
-        test_refpkg.validate()
+        self.assertTrue(test_refpkg.validate())
         self.assertEqual(70, test_refpkg.num_seqs)
         self.assertEqual(2, len(test_refpkg.pfit))
         self.assertTrue(test_refpkg.pfit[0] < 0)
@@ -119,7 +139,7 @@ class TreesappTester(unittest.TestCase):
                                 "--profile", get_test_data("PuhA_search.hmm"),
                                 "--molecule", "prot",
                                 "--screen", "Bacteria,Archaea",
-                                "--num_proc", str(4),
+                                "--num_procs", str(self.num_procs),
                                 "--raxml_model", "LG+F+R4",
                                 "--min_taxonomic_rank", 'p',
                                 "--stage", "support",
@@ -129,7 +149,7 @@ class TreesappTester(unittest.TestCase):
         test_refpkg = ReferencePackage()
         test_refpkg.f__json = "./TreeSAPP_create/final_outputs/PuhA_build.pkl"
         test_refpkg.slurp()
-        test_refpkg.validate()
+        self.assertTrue(test_refpkg.validate())
         self.assertEqual(39, test_refpkg.num_seqs)
         self.assertEqual("LG+F+R4", test_refpkg.sub_model)
         return
@@ -147,14 +167,14 @@ class TreesappTester(unittest.TestCase):
                     "--molecule", "prot",
                     "--screen", "Archaea", "--filter", "Bacteria,Eukaryota",
                     "--min_taxonomic_rank", 'g',
-                    "--num_proc", str(2),
+                    "--num_procs", str(self.num_procs),
                     "--stage", "evaluate",
                     "--trim_align", "--cluster", "--fast", "--headless", "--overwrite", "--delete"]
         create(cmd_list)
         test_refpkg = ReferencePackage()
         test_refpkg.f__json = "./TreeSAPP_create/final_outputs/PF01280_build.pkl"
         test_refpkg.slurp()
-        test_refpkg.validate()
+        self.assertTrue(test_refpkg.validate())
         self.assertEqual(54, test_refpkg.num_seqs)
         return
 
@@ -167,45 +187,17 @@ class TreesappTester(unittest.TestCase):
                                  "-o", "./TreeSAPP_evaluate",
                                  "-m", "prot",
                                  "--taxonomic_ranks", "class", "order",
-                                 "-n", str(2),
+                                 "-n", str(self.num_procs),
                                  "--trim_align", "--overwrite", "--delete"]
         evaluate(evaluate_command_list)
         self.assertEqual(True, True)
         return
 
-    def test_abundance(self):
-        from treesapp.commands import abundance
-        from treesapp.file_parsers import read_classification_table
-        from .testing_utils import get_test_data
-        classification_table = os.path.join(self.ts_assign_output, "final_outputs", "marker_contig_map.tsv")
-        pre_lines = read_classification_table(get_test_data(classification_table))
-        abundance_command_list = ["--treesapp_output", self.ts_assign_output,
-                                  "--reads", get_test_data("test_TarA.1.fq"),
-                                  "--reverse", get_test_data("test_TarA.2.fq"),
-                                  "--pairing", "pe",
-                                  "--num_procs", str(2),
-                                  "--delete"]
-        abundance(abundance_command_list)
-        post_lines = read_classification_table(get_test_data(classification_table))
-        self.assertEqual(len(pre_lines), len(post_lines))
-        return
-
-    def test_purity(self):
-        from treesapp.commands import purity
-        from .testing_utils import get_treesapp_file
-        purity_command_list = ["--fastx_input", get_treesapp_file("dev_utils/TIGRFAM_seed_named.faa"),
-                               "--extra_info", get_treesapp_file("dev_utils/TIGRFAM_info.tsv"),
-                               "--output", "./TreeSAPP_purity",
-                               "--refpkg_path", self.mcra_pkl,
-                               "--trim_align", "--molecule", "prot", "-n", str(2)]
-        purity(purity_command_list)
-        self.assertEqual(True, True)
-        return
-
     def test_layer(self):
         from treesapp.commands import layer
         from .testing_utils import get_test_data
         from treesapp.file_parsers import read_classification_table
+        # Layering annotations from multiple reference packages
         original_table = os.path.join(self.ts_assign_output, "final_outputs", "marker_contig_map.tsv")
         layered_table = os.path.join(self.ts_assign_output, "final_outputs",
                                      "extra_annotated_marker_contig_map.tsv")
@@ -215,12 +207,8 @@ class TreesappTester(unittest.TestCase):
         layer(layer_command_list)
         post_lines = read_classification_table(get_test_data(layered_table))
         self.assertEqual(len(pre_lines), len(post_lines))
-        return
 
-    def test_xmoa_layer(self):
-        from treesapp.commands import layer
-        from .testing_utils import get_test_data
-        from treesapp.file_parsers import read_classification_table
+        # With a different reference package, XmoA, just to be sure
         classification_table = os.path.join(get_test_data("p_amoA_FunGene9.5_isolates_assign/"),
                                             "final_outputs", "marker_contig_map.tsv")
         pre_lines = read_classification_table(get_test_data(classification_table))
@@ -232,46 +220,51 @@ class TreesappTester(unittest.TestCase):
         self.assertEqual(len(pre_lines), len(post_lines))
         return
 
-    def test_update_resolve(self):
-        from treesapp.commands import update
-        from treesapp.refpkg import ReferencePackage
-        from .testing_utils import get_test_data
-        update_command_list = ["--fastx_input", get_test_data("Photosynthesis/PuhA/ENOG4111FIN_PF03967_seed.faa"),
-                               "--refpkg_path", self.puha_pkl,
-                               "--treesapp_output", get_test_data("assign_SwissProt_PuhA/"),
-                               "--output", "./TreeSAPP_update",
-                               "--num_proc", str(4),
-                               "--molecule", "prot",
-                               "--trim_align", "--cluster", "--fast", "--headless",
-                               "--overwrite", "--delete", "--skip_assign", "--resolve"]
-        update(update_command_list)
-        test_refpkg = ReferencePackage()
-        test_refpkg.f__json = "./TreeSAPP_update/final_outputs/PuhA_build.pkl"
-        test_refpkg.slurp()
-        test_refpkg.validate()
-        self.assertEqual(50, test_refpkg.num_seqs)
+    def test_purity(self):
+        from treesapp.commands import purity
+        from .testing_utils import get_treesapp_file
+        purity_command_list = ["--fastx_input", get_treesapp_file("dev_utils/TIGRFAM_seed_named.faa"),
+                               "--extra_info", get_treesapp_file("dev_utils/TIGRFAM_info.tsv"),
+                               "--output", "./TreeSAPP_purity",
+                               "--refpkg_path", self.mcra_pkl,
+                               "--num_procs", str(self.num_procs),
+                               "--trim_align", "--molecule", "prot"]
+        purity(purity_command_list)
+        self.assertEqual(True, True)
         return
 
-    def test_update_seqs2lineage(self):
-        from treesapp.commands import update
+    def test_package(self):
+        from treesapp.commands import package
         from treesapp.refpkg import ReferencePackage
-        from .testing_utils import get_test_data
-        update_command_list = ["--fastx_input", get_test_data("Photosynthesis/PuhA/ENOG4111FIN_PF03967_seed.faa"),
-                               "--refpkg_path", self.puha_pkl,
-                               "--treesapp_output", get_test_data("assign_SwissProt_PuhA/"),
-                               "--seqs2lineage", get_test_data("SwissProt_PuhA_seqs2lineage.txt"),
-                               "--output", "./TreeSAPP_update",
-                               "--num_proc", str(2),
-                               "--molecule", "prot",
-                               "-b", str(0),
-                               "--trim_align", "--cluster", "--fast", "--headless",
-                               "--overwrite", "--delete", "--skip_assign"]
-        update(update_command_list)
+        view_command_list = ["view", "lineage_ids",
+                             "--refpkg_path", self.mcra_pkl,
+                             "--output", "./TreeSAPP_package"]
+        package(view_command_list)
+        edit_command_list = ["edit",
+                             "f__msa", self.aa_test_fa,
+                             "--refpkg_path", self.mcra_pkl,
+                             "--output", "./TreeSAPP_package"]
+        package(edit_command_list)
         test_refpkg = ReferencePackage()
-        test_refpkg.f__json = "./TreeSAPP_update/final_outputs/PuhA_build.pkl"
+        test_refpkg.f__json = "./TreeSAPP_package/McrA_build.pkl"
         test_refpkg.slurp()
-        test_refpkg.validate()
-        self.assertEqual(50, test_refpkg.num_seqs)
+        self.assertFalse(test_refpkg.validate())
+        return
+
+    def test_mcc_calculator(self):
+        from treesapp import MCC_calculator
+        from .testing_utils import get_test_data
+        cmd = ["--fastx_input", get_test_data("EggNOG_McrA.faa"),
+               "--annot_map", get_test_data("EggNOG_McrA_annot_map.tsv"),
+               "--output", "./TreeSAPP_MCC",
+               "--refpkg_dir", self.refpkg_dir,
+               "--targets", "McrA",
+               "--molecule", "prot",
+               "--tool", "treesapp",
+               "--num_procs", str(self.num_procs),
+               "--delete", "--svm", "--overwrite"]
+        MCC_calculator.mcc_calculator(cmd)
+        self.assertEqual(True, True)
         return
 
     def test_train(self):
@@ -286,7 +279,7 @@ class TreesappTester(unittest.TestCase):
                               "--refpkg_path", self.puha_pkl,
                               "--accession2lin", get_test_data("ENOG4111FIN_accession_id_lineage_map.tsv"),
                               "--max_examples", str(max_ex),
-                              "--num_proc", str(4),
+                              "--num_proc", str(self.num_procs),
                               "--molecule", "prot",
                               "--svm_kernel", "rbf",
                               "--classifier", "bin",
@@ -303,37 +296,56 @@ class TreesappTester(unittest.TestCase):
         self.assertEqual(max_ex, len(rank_list))
         return
 
-    def test_package(self):
-        from treesapp.commands import package
-        command_list = ["view",
-                        "lineage_ids",
-                        "--refpkg_path", self.mcra_pkl,
-                        "--output", "./TreeSAPP_package"]
-        package(command_list)
-        self.assertEqual(True, True)
+    def test_update_resolve(self):
+        from treesapp.commands import update
+        from treesapp.refpkg import ReferencePackage
+        from .testing_utils import get_test_data
+        update_command_list = ["--fastx_input", get_test_data("Photosynthesis/PuhA/ENOG4111FIN_PF03967_seed.faa"),
+                               "--refpkg_path", self.puha_pkl,
+                               "--treesapp_output", get_test_data("assign_SwissProt_PuhA/"),
+                               "--output", "./TreeSAPP_update",
+                               "--num_proc", str(self.num_procs),
+                               "--molecule", "prot",
+                               "--trim_align", "--cluster", "--fast", "--headless",
+                               "--overwrite", "--delete", "--skip_assign", "--resolve"]
+        update(update_command_list)
+        test_refpkg = ReferencePackage()
+        test_refpkg.f__json = "./TreeSAPP_update/final_outputs/PuhA_build.pkl"
+        test_refpkg.slurp()
+        self.assertTrue(test_refpkg.validate())
+        self.assertEqual(50, test_refpkg.num_seqs)
         return
 
-    def test_mcc_calculator(self):
-        from treesapp import MCC_calculator
+    def test_update_seqs2lineage(self):
+        from treesapp.commands import update
+        from treesapp.refpkg import ReferencePackage
         from .testing_utils import get_test_data
-        cmd = ["--fastx_input", get_test_data("EggNOG_McrA.faa"),
-               "--annot_map", get_test_data("EggNOG_McrA_annot_map.tsv"),
-               "--output", "./TreeSAPP_MCC",
-               "--refpkg_dir", self.refpkg_dir,
-               "--targets", "McrA",
-               "--molecule", "prot",
-               "--tool", "treesapp",
-               "--num_procs", str(4),
-               "--delete", "--svm", "--overwrite"]
-        MCC_calculator.mcc_calculator(cmd)
-        self.assertEqual(True, True)
+        update_command_list = ["--fastx_input", get_test_data("Photosynthesis/PuhA/ENOG4111FIN_PF03967_seed.faa"),
+                               "--refpkg_path", self.puha_pkl,
+                               "--treesapp_output", get_test_data("assign_SwissProt_PuhA/"),
+                               "--seqs2lineage", get_test_data("SwissProt_PuhA_seqs2lineage.txt"),
+                               "--output", "./TreeSAPP_update",
+                               "--num_procs", str(self.num_procs),
+                               "--molecule", "prot",
+                               "-b", str(0),
+                               "--trim_align", "--cluster", "--fast", "--headless",
+                               "--overwrite", "--delete", "--skip_assign"]
+        update(update_command_list)
+        test_refpkg = ReferencePackage()
+        test_refpkg.f__json = "./TreeSAPP_update/final_outputs/PuhA_build.pkl"
+        test_refpkg.slurp()
+        self.assertTrue(test_refpkg.validate())
+        self.assertEqual(50, test_refpkg.num_seqs)
         return
 
     # def test_tmp(self):
-    #     from treesapp.commands import create
+    #     # from treesapp.commands import create
+    #     from treesapp import MCC_calculator
     #     base_dir = "/home/connor/Bioinformatics/Hallam_projects/RefPkgs/"
-    #     cmd = "".format(base_dir)
-    #     create(cmd.split())
+    #     cmd = "-i /home/connor/Bioinformatics/Hallam_projects/TreeSAPP/tests/test_data/EggNOGv4.5_PF00380_PF00410.faa -o /home/connor/Desktop/TreeSAPP_outputs/MCC_treesapp_0.9.3/ --trim_align -m prot --annot_map /home/connor/Bioinformatics/Hallam_projects/TreeSAPP_manuscript/EggNOG_refpkg_OG_map.tsv -n 8 --targets T0380,T0410 --overwrite"
+    #     MCC_calculator.mcc_calculator(cmd.split())
+    #     # cmd = "".format(base_dir)
+    #     # create(cmd.split())
     #     return
 
 
