@@ -47,20 +47,42 @@ class RefPkgTester(unittest.TestCase):
         self.assertTrue(os.path.isfile(os.path.join(self.disband_path, "McrA.fa")))
 
     def test_remove_taxon_from_lineage_ids(self):
+        # Ensure the initial state is as expected
+        self.assertEqual(246, self.db.num_seqs)
+        self.assertEqual(246, len(self.db.lineage_ids))
+
+        # Test
         self.db.remove_taxon_from_lineage_ids("d__Archaea; p__Euryarchaeota; c__Methanobacteria; o__Methanobacteriales;"
                                               " f__Methanobacteriaceae; g__Methanosphaera")
-        self.assertEqual(190, self.db.num_seqs)
+        self.assertEqual(193, self.db.num_seqs)
         self.db.remove_taxon_from_lineage_ids("d__Archaea; p__Euryarchaeota; c__Methanobacteria")
-        self.assertEqual(143, len(self.db.lineage_ids))
+        self.assertEqual(146, len(self.db.lineage_ids))
 
         self.db.remove_taxon_from_lineage_ids("d__Archaea")
-        self.assertEqual(0, len(self.db.lineage_ids))
-        self.assertEqual(0, self.db.num_seqs)
+        self.assertEqual(3, len(self.db.lineage_ids))
+        self.assertEqual(3, self.db.num_seqs)
 
     def test_get_fasta(self):
         refpkg_fa = self.db.get_fasta()
         self.assertEqual(246, len(refpkg_fa.fasta_dict))
 
+    def test_get_ete_tree(self):
+        from treesapp.refpkg import ReferencePackage
+        rt = self.db.get_ete_tree()
+        self.assertEqual(246, len(rt))
+        blank = ReferencePackage()
+        with pytest.raises(SystemExit):
+            blank.get_ete_tree()
+        return
+
+    def test_hmm_length(self):
+        from treesapp.refpkg import ReferencePackage
+        blank = ReferencePackage()
+        self.assertEqual(blank.profile, [])
+        with pytest.raises(ValueError):
+            blank.hmm_length()
+        return
+
     def test_bail(self):
         self.db.bail()
 
@@ -69,8 +91,9 @@ class RefPkgTester(unittest.TestCase):
         self.db.f__json = utils.get_test_data(os.path.join("refpkgs", "McrA_build.pkl"))
         self.db.slurp()
         node_map = self.db.get_internal_node_leaf_map()
-        self.assertEqual((2*self.db.num_seqs)-1, len(node_map))
+        self.assertEqual(2*self.db.num_seqs, len(node_map))
         self.assertEqual(self.db.num_seqs, len(node_map[max(node_map.keys())]))
+        return
 
     def test_pickle_package(self):
         self.db.f__json = self.new_pkl_path
@@ -78,6 +101,25 @@ class RefPkgTester(unittest.TestCase):
         self.db.slurp()
         self.assertTrue("McrA" == self.db.prefix)
 
+    def test_taxonomically_label_tree(self):
+        from treesapp.taxonomic_hierarchy import Taxon
+        labelled_rt = self.db.taxonomically_label_tree()
+        self.assertEqual(246, len(labelled_rt))
+        self.assertTrue('taxon' in labelled_rt.features)
+        self.assertEqual('Root', labelled_rt.taxon.name)
+        self.assertEqual('root', labelled_rt.taxon.rank)
+        self.assertTrue(isinstance(labelled_rt.taxon, Taxon))
+        return
+
+    def test_enumerate_taxonomic_lineages(self):
+        from treesapp.refpkg import ReferencePackage
+        mock_rp = ReferencePackage()
+        self.assertEqual(0, len(mock_rp.enumerate_taxonomic_lineages()))
+        taxa_counts = self.db.enumerate_taxonomic_lineages()
+        self.assertIsInstance(taxa_counts, dict)
+        self.assertTrue("r__Root" in taxa_counts)
+        return
+
 
 if __name__ == '__main__':
     unittest.main()
