@@ -8,7 +8,6 @@ import logging
 import time
 from datetime import datetime as dt
 from shutil import rmtree, copy
-from multiprocessing import Process
 from glob import glob
 from collections import namedtuple
 from numpy import var
@@ -17,13 +16,10 @@ from treesapp.phylo_seq import convert_entrez_to_tree_leaf_references, PQuery
 from treesapp.refpkg import ReferencePackage
 from treesapp.fasta import fastx_split, get_header_format, FASTA, load_fasta_header_regexes, sequence_info_groups
 from treesapp.utilities import median, write_dict_to_table, validate_new_dir, fetch_executable_path
-from treesapp.entish import create_tree_info_hash, subtrees_to_dictionary
 from treesapp.lca_calculations import determine_offset, optimal_taxonomic_assignment
 from treesapp import entrez_utils
 from treesapp.wrapper import CommandLineFarmer, estimate_ml_model
 
-import _tree_parser
-
 
 class ModuleFunction:
     def __init__(self, name: str, order: int, func=None):
@@ -46,28 +42,28 @@ class ModuleFunction:
         return info_string
 
 
-class NodeRetrieverWorker(Process):
-    """
-    Doug Hellman's Consumer class for handling processes via queues
-    """
-
-    def __init__(self, task_queue, result_queue):
-        Process.__init__(self)
-        self.task_queue = task_queue
-        self.result_queue = result_queue
-
-    def run(self):
-        while True:
-            next_task = self.task_queue.get()
-            if next_task is None:
-                # Poison pill means shutdown
-                self.task_queue.task_done()
-                break
-            result = _tree_parser._build_subtrees_newick(next_task)
-            subtrees = subtrees_to_dictionary(result, create_tree_info_hash())
-            self.task_queue.task_done()
-            self.result_queue.put(subtrees)
-        return
+# class NodeRetrieverWorker(Process):
+#     """
+#     Doug Hellman's Consumer class for handling processes via queues
+#     """
+#
+#     def __init__(self, task_queue, result_queue):
+#         Process.__init__(self)
+#         self.task_queue = task_queue
+#         self.result_queue = result_queue
+#
+#     def run(self):
+#         while True:
+#             next_task = self.task_queue.get()
+#             if next_task is None:
+#                 # Poison pill means shutdown
+#                 self.task_queue.task_done()
+#                 break
+#             result = _tree_parser._build_subtrees_newick(next_task)
+#             subtrees = subtrees_to_dictionary(result, create_tree_info_hash())
+#             self.task_queue.task_done()
+#             self.result_queue.put(subtrees)
+#         return
 
 
 def get_header_info(header_registry: dict, code_name=''):
@@ -341,10 +337,10 @@ class TreeSAPP:
                 self.formatted_input = self.var_output_dir + self.sample_prefix + "_formatted.fasta"
 
         if self.command != "colour" and "pkg_path" in vars(args):
-            args.pkg_path = args.pkg_path.pop(0)
             if len(args.pkg_path) > 1:
                 logging.warning("Multiple reference packages cannot be used by treesapp {}.\n"
                                 "Only the first one ({}) will be used.\n".format(self.command, args.pkg_path))
+            args.pkg_path = args.pkg_path.pop(0)
 
         self.executables = self.find_executables(args)
         return
@@ -930,8 +926,7 @@ class Purity(TreeSAPP):
             seq_info = re.match(r"(.*)\|" + re.escape(p_query.ref_name) + r"\|(\\d+)_(\\d+)$", p_query.place_name)
             if seq_info:
                 p_query.place_name = seq_info.group(1)
-            p_query.inode = int(p_query.get_jplace_element("edge_num"))
-            leaves = internal_node_map[p_query.inode]
+            leaves = internal_node_map[p_query.consensus_placement.edge_num]
             ortholog_name = p_query.place_name.split('_')[0]
             if ortholog_name not in ortholog_map:
                 ortholog_map[ortholog_name] = []
@@ -957,8 +952,8 @@ class TaxonTest:
         self.classifier_output = ""
 
     def get_optimal_assignment(self):
-        if self.lineage.split('; ')[0] != "Root":
-            self.lineage = "; ".join(["Root"] + self.lineage.split("; "))
+        if self.lineage.split('; ')[0] != "r__Root":
+            self.lineage = "; ".join(["r__Root"] + self.lineage.split("; "))
         return optimal_taxonomic_assignment(self.taxonomic_tree, self.lineage)
 
     def summarise_taxon_test(self):
