@@ -34,7 +34,7 @@ from treesapp.assign import abundify_tree_saps, delete_files, prep_reference_pac
     multiple_alignments, get_sequence_counts, check_for_removed_sequences, determine_confident_lineage,\
     evaluate_trimming_performance, parse_raxml_output, filter_placements, align_reads_to_nucs, select_query_placements,\
     summarize_placements_rpkm, write_classification_table, produce_itol_inputs, replace_contig_names,\
-    read_refpkg_tax_ids, load_homologs, load_pqueries
+    load_homologs, load_pqueries, write_classified_sequences
 from treesapp.jplace_utils import sub_indices_for_seq_names_jplace, jplace_parser, demultiplex_pqueries
 from treesapp.clade_exclusion_evaluator import pick_taxonomic_representatives, select_rep_seqs,\
     map_seqs_to_lineages, prep_graftm_ref_files, build_graftm_package, map_headers_to_lineage, graftm_classify,\
@@ -54,6 +54,8 @@ def info(sys_args):
     import treesapp
     import Bio
     import numpy
+    import packaging
+    import pygtrie
     import scipy
     import ete3
     import sklearn
@@ -61,6 +63,7 @@ def info(sys_args):
     import seaborn
     import samsum
     import pyfastx
+    import tqdm
     logging.info("TreeSAPP version " + treesapp.__version__ + ".\n")
 
     # Write the version of all python deps
@@ -68,11 +71,14 @@ def info(sys_args):
                "ete3": ete3.__version__,
                "joblib": joblib.__version__,
                "numpy": numpy.__version__,
-               "scipy": scipy.__version__,
-               "scikit-learn": sklearn.__version__,
-               "seaborn": seaborn.__version__,
+               "packaging": packaging.__version__,
+               "pyfastx": pyfastx.version(),
+               "pygtrie": pygtrie.__version__,
                "samsum": samsum.__version__,
-               "pyfastx": pyfastx.version()}
+               "scikit-learn": sklearn.__version__,
+               "scipy": scipy.__version__,
+               "seaborn": seaborn.__version__,
+               "tqdm": tqdm.__version__}
 
     logging.info("Python package dependency versions:\n\t" +
                  "\n\t".join([k + ": " + v for k, v in py_deps.items()]) + "\n")
@@ -180,8 +186,8 @@ def train(sys_args):
         logging.info("done.\n")
         ts_trainer.increment_stage_dir()
     else:
-        if not os.path.isfile(ts_trainer.formatted_input):
-            ts_trainer.formatted_input = ts_trainer.input_sequences
+        ts_trainer.formatted_input = ts_trainer.input_sequences
+        train_seqs.header_registry = fasta.register_headers(fasta.get_headers(ts_trainer.formatted_input), True)
     train_seqs.file = ts_trainer.formatted_input
 
     ##
@@ -207,6 +213,9 @@ def train(sys_args):
             ts_trainer.hmm_purified_seqs = ts_trainer.input_sequences
             train_seqs.file = ts_trainer.hmm_purified_seqs
             train_seqs.fasta_dict = fasta.read_fasta_to_dict(train_seqs.file)
+            if not train_seqs.fasta_dict:
+                logging.error("No sequences were detected in the HMM-purified FASTA '{}'.\n".format(train_seqs.file))
+                sys.exit(13)
         else:
             train_seqs.file = ts_trainer.hmm_purified_seqs
             train_seqs.load_fasta()
@@ -321,7 +330,8 @@ def train(sys_args):
                 if not pquery.rank:
                     pquery.rank = "species"
         else:
-            tp, fp, fn = bin_headers(refpkg_pqueries, ts_trainer.pkg_dbname_dict, query_seq_records)
+            tp, fp, fn = bin_headers(refpkg_pqueries, ts_trainer.pkg_dbname_dict, query_seq_records,
+                                     {ts_trainer.ref_pkg.prefix: ts_trainer.ref_pkg})
             # The individual instances are of QuerySequence type
             for refpkg_name, tp_query_seqs in tp.items():
                 tp_names[refpkg_name] = [qseq.place_name for qseq in tp_query_seqs]
@@ -467,9 +477,8 @@ def create(sys_args):
         # Remove the sequences failing 'filter' and/or only retain the sequences in 'screen'
         fasta_records = create_refpkg.screen_filter_taxa(fasta_records, args.screen, args.filter, ref_seqs.amendments)
         # Remove the sequence records with low resolution lineages, according to args.min_taxonomic_rank
-        # TODO: Replace this function with one offered by TaxonomicHierarchy
-        fasta_records = create_refpkg.remove_by_truncated_lineages(fasta_records,
-                                                                   args.min_taxonomic_rank, ref_seqs.amendments)
+        fasta_records = create_refpkg.remove_by_truncated_lineages(fasta_records, args.min_taxonomic_rank,
+                                                                   ts_create.ref_pkg.taxa_trie, ref_seqs.amendments)
 
         if len(fasta_records.keys()) < 2:
             logging.error("{} sequences post-homology + taxonomy filtering\n".format(len(fasta_records)))
@@ -596,9 +605,9 @@ def create(sys_args):
             wrapper.build_hmm_profile(ts_create.executables["hmmbuild"],
                                       ts_create.ref_pkg.f__msa,
                                       ts_create.ref_pkg.f__profile)
-            ts_create.ref_pkg.hmm_length()
 
         ts_create.ref_pkg.band()
+        ts_create.ref_pkg.hmm_length()
         ts_create.ref_pkg.dereplicate_hmm(dereplication_rank="genus",
                                           hmmbuild_exe=ts_create.executables["hmmbuild"],
                                           mafft_exe=ts_create.executables["mafft"],
@@ -1162,7 +1171,6 @@ def assign(sys_args):
     refpkg_dict = file_parsers.gather_ref_packages(ts_assign.refpkg_dir, ts_assign.target_refpkgs)
     prep_reference_packages_for_assign(refpkg_dict, ts_assign.var_output_dir)
     ref_alignment_dimensions = get_alignment_dims(refpkg_dict)
-    tree_numbers_translation = read_refpkg_tax_ids(refpkg_dict)
 
     ##
     # STAGE 2: Predict open reading frames (ORFs) if the input is an assembly, read, format and write the FASTA
@@ -1250,10 +1258,11 @@ def assign(sys_args):
         sub_indices_for_seq_names_jplace(ts_assign.var_output_dir, numeric_contig_index, refpkg_dict)
 
     if ts_assign.stage_status("classify"):
+        itol_out_dir = ts_assign.output_dir + 'iTOL_output' + os.sep
         tree_saps, itol_data = parse_raxml_output(ts_assign.var_output_dir, refpkg_dict, pqueries)
-        select_query_placements(tree_saps)
-        filter_placements(tree_saps, refpkg_dict, ts_assign.svc_filter, args.min_likelihood)
-        fasta.write_classified_sequences(tree_saps, extracted_seq_dict, ts_assign.classified_aa_seqs)
+        select_query_placements(tree_saps, refpkg_dict, mode=args.p_sum)  # Set PQuery.consensus_placement attributes
+        filter_placements(tree_saps, refpkg_dict, ts_assign.svc_filter, args.min_lwr)
+        write_classified_sequences(tree_saps, extracted_seq_dict, ts_assign.classified_aa_seqs)
         abundance_dict = dict()
         for refpkg_code in tree_saps:
             for placed_seq in tree_saps[refpkg_code]:  # type: PQuery
@@ -1266,7 +1275,7 @@ def assign(sys_args):
                 if not os.path.isfile(ts_assign.classified_nuc_seqs):
                     logging.info("Creating nucleotide FASTA file of classified sequences '" +
                                  ts_assign.classified_nuc_seqs + "'... ")
-                    fasta.write_classified_sequences(tree_saps, nuc_orfs.fasta_dict, ts_assign.classified_nuc_seqs)
+                    write_classified_sequences(tree_saps, nuc_orfs.fasta_dict, ts_assign.classified_nuc_seqs)
                     logging.info("done.\n")
             else:
                 logging.warning("Unable to read '" + ts_assign.nuc_orfs_file + "'.\n" +
@@ -1283,9 +1292,10 @@ def assign(sys_args):
 
         abundify_tree_saps(tree_saps, abundance_dict)
         assign_out = ts_assign.final_output_dir + os.sep + "marker_contig_map.tsv"
-        determine_confident_lineage(tree_saps, tree_numbers_translation, refpkg_dict)
+        determine_confident_lineage(tree_saps, refpkg_dict)
         write_classification_table(tree_saps, ts_assign.sample_prefix, assign_out)
-        produce_itol_inputs(tree_saps, refpkg_dict, itol_data, ts_assign.output_dir, ts_assign.refpkg_dir)
+
+        produce_itol_inputs(tree_saps, refpkg_dict, itol_data, itol_out_dir, ts_assign.refpkg_dir)
         delete_files(args.delete, ts_assign.var_output_dir, 4)
 
     delete_files(args.delete, ts_assign.var_output_dir, 5)
@@ -1419,10 +1429,12 @@ def purity(sys_args):
         jplace_file = os.sep.join([ts_purity.assign_dir, "iTOL_output", ts_purity.ref_pkg.prefix,
                                    ts_purity.ref_pkg.prefix + "_complete_profile.jplace"])
         jplace_data = jplace_parser(jplace_file)
-        tree_placement_queries = demultiplex_pqueries(jplace_data)
-        placement_tree = jplace_data.tree
-        node_map = entish.map_internal_nodes_leaves(placement_tree)
-        ortholog_map = ts_purity.assign_leaves_to_orthologs(tree_placement_queries, node_map)
+        jplace_data.pqueries = demultiplex_pqueries(jplace_data)
+        node_map = entish.map_internal_nodes_leaves(jplace_data.tree)
+        labelled_ref_tree = ts_purity.ref_pkg.taxonomically_label_tree()
+        for pquery in jplace_data.pqueries:  # type: PQuery
+            pquery.process_max_weight_placement(labelled_ref_tree)
+        ortholog_map = ts_purity.assign_leaves_to_orthologs(jplace_data.pqueries, node_map)
         ts_purity.summarize_groups_assigned(ortholog_map, metadat_dict)
 
         # Write each sequence name that can be assigned to an ortholog to the log
