@@ -2,7 +2,6 @@
 
 import os
 import sys
-import argparse
 import logging
 import re
 import random
@@ -13,64 +12,34 @@ from treesapp import file_parsers
 from treesapp import utilities
 from treesapp import wrapper
 from treesapp import fasta
-from treesapp.phylo_seq import JPlace, PQuery
+from treesapp.phylo_seq import PQuery
 from treesapp.phylo_dist import cull_outliers, regress_ranks
 from treesapp.taxonomic_hierarchy import TaxonomicHierarchy
 from treesapp.refpkg import ReferencePackage
 from treesapp.training_utils import rarefy_rank_distances, generate_pquery_data_for_trainer
 
-__author__ = 'Connor Morgan-Lang'
+
+def fail_training(msg) -> None:
+    boilerplate = "Clade-exclusion analysis could not be performed for training the reference package models:\n"
+    logging.error(boilerplate + msg + "\n")
+    return
 
 
-def get_options():
-    parser = argparse.ArgumentParser(description="Workflow for estimating calibrating the edge distances corresponding"
-                                                 " to taxonomic ranks by iterative leave-one-out validation")
-    required_args = parser.add_argument_group("Required arguments")
-    seqop_args = parser.add_argument_group("Sequence operation arguments")
-    taxa_args = parser.add_argument_group("Taxonomic-lineage arguments")
-    miscellaneous_opts = parser.add_argument_group("Miscellaneous arguments")
-
-    required_args.add_argument("-f", "--fasta_input", required=True,
-                               help='The raw, unclustered and unfiltered FASTA file to train the reference package.')
-    required_args.add_argument("-n", "--name", required=True,
-                               help="Prefix name of the reference package "
-                                    "(i.e. McrA for McrA.fa, McrA.hmm, McrA_tree.txt)")
-    required_args.add_argument("-p", "--pkg_path", required=True,
-                               help="The path to the TreeSAPP-formatted reference package.")
-    seqop_args.add_argument("-d", "--domain",
-                            help="An HMM profile representing a specific domain.\n"
-                                 "Domains will be excised from input sequences based on hmmsearch alignments.",
-                            required=False, default=None)
-    taxa_args.add_argument("-l", "--lineages",
-                           help="The accession lineage map downloaded during reference package generation.",
-                           required=False)
-    miscellaneous_opts.add_argument('-m', '--molecule', default='prot', choices=['prot', 'dna', 'rrna'],
-                                    help='the type of input sequences (prot = Protein [DEFAULT]; dna = Nucleotide )')
-    miscellaneous_opts.add_argument("-T", "--num_threads", required=False, default=4, type=int,
-                                    help="The number of threads to be used by RAxML.")
-    miscellaneous_opts.add_argument("-o", "--output_dir", required=False, default='.',
-                                    help="Path to directory for writing outputs.")
-    miscellaneous_opts.add_argument("-v", "--verbose", action='store_true',  default=False,
-                                    help='Prints a more verbose runtime log')
-    miscellaneous_opts.add_argument("-O", "--overwrite", default=False, action="store_true",
-                                    help="Force recalculation of placement distances for query sequences.")
-    args = parser.parse_args()
-    args.targets = ["ALL"]
-    return args
-
-
-def write_placement_table(pqueries: dict, placement_table_file, marker):
+def write_placement_table(pqueries: dict, placement_table_file: str, marker: str) -> None:
     header = ["Marker", "Rank", "Lineage", "Query.Name", "Internal.Node", "Placement.LWR", "Tree.Likelihood",
               "Dist.Distal", "Dist.Pendant", "Dist.MeanTip", "Dist.Total"]
     placement_info_strs = list()
     for rank, taxa in pqueries.items():
         for taxon in taxa:
-            for pquery in taxa[taxon]:
+            for pquery in taxa[taxon]:  # type: PQuery
                 if pquery:
                     placement_info_strs.append("\t".join(
-                        [marker, str(pquery.rank), str(pquery.lineage), str(pquery.ref_name), str(pquery.inode),
-                         str(pquery.lwr), str(pquery.likelihood),
-                         str(pquery.distal), str(pquery.pendant), str(pquery.mean_tip), str(pquery.total_distance())])
+                        [marker, str(pquery.rank), str(pquery.lineage),
+                         str(pquery.ref_name), str(pquery.consensus_placement.edge_num),
+                         str(pquery.consensus_placement.like_weight_ratio), str(pquery.consensus_placement.likelihood),
+                         str(pquery.consensus_placement.distal_length), str(pquery.consensus_placement.pendant_length),
+                         str(pquery.consensus_placement.mean_tip_length),
+                         str(pquery.consensus_placement.total_distance())])
                     )
 
     with open(placement_table_file, 'w') as file_handler:
@@ -97,7 +66,7 @@ def flatten_pquery_dict(pqueries: dict, refpkg_prefix: str) -> dict:
                 for pquery in taxa[taxon]:  # type: PQuery
                     refpkg_pqueries[refpkg_prefix].append(pquery)
             except TypeError:
-                if isinstance(taxon, JPlace):
+                if isinstance(taxon, PQuery):
                     refpkg_pqueries[refpkg_prefix].append(taxon)
                 else:
                     logging.error("An instance of type PQuery was expected, found '{}' instead.\n".format(type(taxon)))
@@ -222,9 +191,11 @@ def prepare_training_data(test_seqs: fasta.FASTA, output_dir: str, executables:
 
     # Remove sequences that are not related at the rank of Domain
     ref_domains = t_hierarchy.rank_representatives("domain", True)
-    for seq_name in sorted(accession_lineage_map):
-        query_domain = accession_lineage_map[seq_name].split(lin_sep)[0]
-        if query_domain not in ref_domains:
+    for seq_name in sorted(accession_lineage_map):  # type: str
+        query_domain = t_hierarchy.get_rank_from_lineage(accession_lineage_map[seq_name].split(lin_sep)[-1], "domain")
+        if not query_domain:
+            continue
+        elif query_domain.prefix_taxon() not in ref_domains:
             unrelated_queries.append(seq_name)
         else:
             related_queries.append(seq_name)
@@ -241,8 +212,11 @@ def prepare_training_data(test_seqs: fasta.FASTA, output_dir: str, executables:
     # Calculate the number of sequences that cannot be used in clade exclusion analysis due to no coverage in the input
     test_taxa_summary = []
     for rank in taxonomic_ranks:
-        test_taxa_summary.append("Sequences available for training %s-level placement distances:" % rank)
         trimmed_ref_lineages = t_hierarchy.trim_lineages_to_rank(leaf_taxa_map, rank)
+        if not trimmed_ref_lineages:
+            logging.warning("No reference sequences are resolved to the rank '{}' in reference package.\n".format(rank))
+            continue
+        test_taxa_summary.append("Sequences available for training %s-level placement distances:" % rank)
         unique_ref_lineages = sorted(set(trimmed_ref_lineages.values()))
 
         # Remove all sequences belonging to a taxonomic rank from tree and reference alignment
@@ -337,8 +311,7 @@ def prepare_training_data(test_seqs: fasta.FASTA, output_dir: str, executables:
 
 def clade_exclusion_phylo_placement(rank_training_seqs: dict,
                                     test_fasta: fasta.FASTA, ref_pkg: ReferencePackage,
-                                    leaf_taxa_map: dict, executables: dict,
-                                    output_dir="./", raxml_threads=4) -> dict:
+                                    executables: dict, output_dir="./", raxml_threads=4, min_seqs=30) -> dict:
     """
     Function for iteratively performing leave-one-out analysis for every taxonomic lineage represented in the tree,
     yielding an estimate of placement distances corresponding to taxonomic ranks.
@@ -347,14 +320,13 @@ def clade_exclusion_phylo_placement(rank_training_seqs: dict,
      to rank depth values where Kingdom is 0, Phylum is 1, etc.
     :param test_fasta: Dictionary with headers as keys and sequences as values for deduplicated training sequences
     :param ref_pkg: A ReferencePackage instance
-    :param leaf_taxa_map: A dictionary mapping TreeSAPP numeric sequence identifiers to taxonomic lineages
     :param executables: A dictionary mapping software to a path of their respective executable
     :param output_dir: Path to directory where all intermediate files should be written
     :param raxml_threads: Number of threads to be used by RAxML for parallel computation
+    :param min_seqs: The minimum number of sequences required for training all taxonomic ranks of a reference package
 
     :return: Dictionary of ranks indexing a dictionary of taxa of that rank indexing a list of PQuery instances
     """
-    leaf_trimmed_taxa_map = dict()
     pqueries = dict()
 
     if output_dir[-1] != os.sep:
@@ -373,8 +345,8 @@ def clade_exclusion_phylo_placement(rank_training_seqs: dict,
             logging.debug("{} sequences to train {}-level placement distances\n".format(num_rank_training_seqs, rank))
         num_training_queries += num_rank_training_seqs
 
-    if num_training_queries < 30:
-        logging.error("Too few ({}) sequences for training placement distance model.\n".format(num_training_queries))
+    if num_training_queries < min_seqs:
+        fail_training("Too few ({}) sequences for training placement distance model.\n".format(num_training_queries))
         return pqueries
     if num_training_queries < 50:
         logging.warning("Only {} sequences for training placement distance model.\n".format(num_training_queries))
@@ -386,9 +358,6 @@ def clade_exclusion_phylo_placement(rank_training_seqs: dict,
     for rank in sorted(rank_training_seqs, reverse=True):
         pbar.set_description("Processing %s" % rank)
         pqueries[rank] = {}
-        for leaf_node, lineage in ref_pkg.taxa_trie.trim_lineages_to_rank(leaf_taxa_map, rank).items():
-            leaf_trimmed_taxa_map[leaf_node + "_" + ref_pkg.prefix] = lineage
-
         for taxonomy in sorted(rank_training_seqs[rank]):
             logging.debug("Testing placements for {}:\n".format(taxonomy))
             pqueries[rank][taxonomy] = generate_pquery_data_for_trainer(ref_pkg, taxonomy, test_fasta,
@@ -398,7 +367,6 @@ def clade_exclusion_phylo_placement(rank_training_seqs: dict,
         if len(pqueries[rank]) == 0:
             logging.debug("No samples available for " + rank + ".\n")
 
-        leaf_trimmed_taxa_map.clear()
     pbar.close()
 
     return pqueries
@@ -419,19 +387,22 @@ def evo_dists_from_pqueries(pqueries: dict, training_ranks=None) -> dict:
         training_ranks = {"class": 3, "species": 7}
 
     # Populate dictionary of evolutionary distances indexed by rank and taxon
-    for rank in training_ranks:
+    for rank in training_ranks:  # type: str
+        if rank not in pqueries:
+            fail_training(msg="No phylogenetic placements were performed for taxonomic rank '{}'.\n"
+                              "Either find more reference sequences or increase percent similarity to "
+                              "increase the chances of having multiple representatives of the same rank".format(rank))
+            return {}
+
         taxonomic_placement_distances[rank] = []
         if rank in pqueries and len(pqueries[rank]) > 0:
             for taxon in pqueries[rank]:
-                taxonomic_placement_distances[rank] += [pquery.total_distance() for pquery in pqueries[rank][taxon]]
-        else:
-            logging.warning("Clade-exclusion could not be performed for '{}'.\n"
-                            "No phylogenetic placement data was generated for training.\n".format(rank))
-            continue
+                taxonomic_placement_distances[rank] += [pquery.consensus_placement.total_distance() for pquery
+                                                        in pqueries[rank][taxon]]
 
         if len(taxonomic_placement_distances[rank]) == 0:
-            logging.error("No PQuery distances were transferred for rank {}.\n".format(rank))
-            sys.exit(17)
+            fail_training(msg="No phylogenetic placements succeeded for taxonomic rank '{}'.\n".format(rank))
+            return {}
 
         median_dist = round(utilities.median(taxonomic_placement_distances[rank]), 4)
         mean_dist = round(utilities.mean(taxonomic_placement_distances[rank]), 4)
@@ -522,6 +493,15 @@ def gen_cladex_data(fasta_input: str, executables: dict, ref_pkg: ReferencePacka
     ref_pkg.load_taxonomic_hierarchy()
     for ref_seq in ref_pkg.generate_tree_leaf_references_from_refpkg():
         leaf_taxa_map[ref_seq.number] = ref_seq.lineage
+
+    # Fix the taxonomic lineages of the query sequences in accession_lineage_map
+    for seq_id, lineage in accession_lineage_map.items():
+        taxon_name = lineage.split(ref_pkg.taxa_trie.lin_sep)[-1]
+        if ref_pkg.taxa_trie.emit(taxon_name):
+            accession_lineage_map[seq_id] = ref_pkg.taxa_trie.emit(taxon_name, with_prefix=True)
+        else:
+            pass
+
     # Load the query FASTA and unalign the sequences, in case the fasta is a MSA
     test_seqs = fasta.FASTA(fasta_input)
     test_seqs.load_fasta()
@@ -537,7 +517,7 @@ def gen_cladex_data(fasta_input: str, executables: dict, ref_pkg: ReferencePacka
         return {}
 
     # Perform the rank-wise clade exclusion analysis for estimating placement distances
-    pqueries = clade_exclusion_phylo_placement(rank_training_seqs, test_seqs, ref_pkg, leaf_taxa_map,
+    pqueries = clade_exclusion_phylo_placement(rank_training_seqs, test_seqs, ref_pkg,
                                                executables, output_dir, num_threads)
 
     return pqueries
