@@ -11,11 +11,11 @@ from ete3 import Tree
 import joblib
 
 from treesapp.phylo_seq import TreeLeafReference
-from treesapp.entish import annotate_partition_tree
+from treesapp.entish import annotate_partition_tree, label_internal_nodes_ete, verify_bifurcations
 from treesapp.external_command_interface import launch_write_command
 from treesapp.fasta import read_fasta_to_dict, write_new_fasta, multiple_alignment_dimensions, FASTA, register_headers
-from treesapp.taxonomic_hierarchy import TaxonomicHierarchy
-from treesapp.utilities import get_hmm_length, base_file_prefix, load_taxonomic_trie, match_file
+from treesapp.taxonomic_hierarchy import TaxonomicHierarchy, Taxon
+from treesapp.utilities import base_file_prefix, load_taxonomic_trie, match_file, get_hmm_value
 from treesapp import wrapper
 from treesapp import __version__ as ts_version
 
@@ -88,6 +88,8 @@ class ReferencePackage:
     def get_info(self):
         return "\n\t".join(["ReferencePackage instance of {} ({}):".format(self.prefix, self.refpkg_code),
                             "Molecule type:                                      '{}'".format(self.molecule),
+                            "TreeSAPP version:                                   '{}'".format(self.ts_version),
+                            "Profile HMM length:                                 '{}'".format(self.profile_length),
                             "Substitution model used for phylogenetic inference: '{}'".format(self.sub_model),
                             "Number of reference sequences (leaf nodes):          {}".format(self.num_seqs),
                             "Software used to infer phylogeny:                   '{}'".format(self.tree_tool),
@@ -102,6 +104,7 @@ class ReferencePackage:
         :return: None
         """
         logging.error(msg + "\n" + self.get_info())
+        return
 
     def clone(self, clone_path: str):
         refpkg_clone = ReferencePackage()
@@ -297,8 +300,8 @@ class ReferencePackage:
             return False
 
         # Compare the number of sequences in the multiple sequence alignment
-        refpkg_fa = self.get_fasta()
-        if not refpkg_fa:
+        refpkg_fa = self.get_fasta()  # type: FASTA
+        if refpkg_fa.n_seqs() == 0:
             return False
         if self.num_seqs != refpkg_fa.n_seqs():
             self.bail("Number of sequences in {} "
@@ -307,8 +310,25 @@ class ReferencePackage:
                                                                                      refpkg_fa.n_seqs()))
             return False
 
-        # TODO: Compare the number of sequences in the Hidden-Markov model
-        # TODO: Compare the number of sequences in the Tree files
+        # Compare the number of sequences in the Hidden-Markov model
+        hmm_nseq = self.num_hmm_seqs()
+        if hmm_nseq != self.num_seqs:
+            self.bail("Number of sequences in {} "
+                      "ReferencePackage.num_seqs ({}) and HMM ({}) differ.\n".format(self.prefix,
+                                                                                     self.num_seqs,
+                                                                                     hmm_nseq))
+            return False
+
+        # Compare the number of sequences in the Tree files
+        rt = self.get_ete_tree()
+        if len(rt) != self.num_seqs:
+            self.bail("Number of sequences in {} "
+                      "ReferencePackage.num_seqs ({}) and tree ({}) differ.\n".format(self.prefix,
+                                                                                      self.num_seqs,
+                                                                                      len(rt)))
+            return False
+        self.tree = verify_bifurcations(self.tree)
+
         # Compare the number of sequences in lineage IDs
         n_leaf_nodes = len(self.generate_tree_leaf_references_from_refpkg())
         if n_leaf_nodes != self.num_seqs:
@@ -330,14 +350,16 @@ class ReferencePackage:
 
         :return: A FASTA instance populated by the reference package's msa attribute
         """
+        refpkg_fa = FASTA(self.f__msa)
         if not self.msa:
             logging.debug("ReferencePackage MSA hasn't been slurped, unable to read FASTA.\n")
-            return
+            return refpkg_fa
 
-        refpkg_fa = FASTA(self.f__msa)
         name, seq = "", ""
         for line in self.msa:
-            line = line[:-1]  # Strip the newline character
+            line = line.rstrip()  # Strip the newline character
+            if not line:
+                continue
             if line[0] == '>':
                 if name:  # This is the first sequence
                     refpkg_fa.fasta_dict[name] = seq
@@ -354,20 +376,37 @@ class ReferencePackage:
 
         return refpkg_fa
 
-    def get_internal_node_leaf_map(self):
+    def get_ete_tree(self) -> Tree:
+        if not self.tree:
+            logging.error("Unable to load tree - '{}' reference package hasn't been slurped yet.\n".format(self.prefix))
+            sys.exit(5)
+        if type(self.tree) is list:
+            if len(self.tree) > 0:
+                self.tree = self.tree.pop(0)
+
+        rt = Tree(self.tree)
+        label_internal_nodes_ete(rt)
+        return rt
+
+    def get_internal_node_leaf_map(self) -> dict:
         node_map = dict()
         leaf_stack = list()
+        rt = self.get_ete_tree()
+
         i = 0
-        rt = Tree(self.tree)
         for inode in rt.traverse(strategy="postorder"):
-            if inode.name:
-                node_map[i] = [inode.name]
-                leaf_stack.append(node_map[i])
-            else:
+            if inode.children:
                 node_map[i] = leaf_stack.pop() + leaf_stack.pop()
                 leaf_stack.append(node_map[i])
+            else:
+                node_map[i] = [inode.name]
+                leaf_stack.append(node_map[i])
             i += 1
-        node_map[i] = leaf_stack.pop() + leaf_stack.pop()
+        node_map[i] = leaf_stack.pop()
+        if leaf_stack:
+            logging.error("Some leaves remain unpopped from the stack - internal node to leaf map is incomplete.\n")
+            sys.exit(17)
+
         return node_map
 
     def leaf_node_order(self) -> list:
@@ -376,7 +415,7 @@ class ReferencePackage:
 
         :return: A list of internal node identifiers sorted by post-order traversal
         """
-        tree_root = Tree(self.tree)
+        tree_root = self.get_ete_tree()
         leaf_order = []
         for node in tree_root.traverse(strategy="postorder"):
             if node.name:
@@ -436,13 +475,20 @@ class ReferencePackage:
 
         return
 
-    def hmm_length(self) -> None:
-        self.profile_length = get_hmm_length(self.f__profile)
+    def hmm_length(self) -> int:
+        if not self.profile:
+            self.bail("ReferencePackage attribute 'profile' was not set when trying to find profile HMM length.\n")
+            raise ValueError
+        self.profile_length = int(get_hmm_value(self.profile, "length"))
+        return self.profile_length
+
+    def num_hmm_seqs(self) -> int:
+        return int(get_hmm_value(self.profile, "num_seqs"))
 
     def alignment_dims(self):
         return multiple_alignment_dimensions(self.f__msa)
 
-    def filter_refs_by_lineage(self, target_lineage: str) -> list:
+    def screen_refs_by_lineage(self, target_lineage: str) -> list:
         """
         Returns the TreeLeafReference instances from the reference package that match or are children of target_lineage
 
@@ -456,6 +502,12 @@ class ReferencePackage:
                 children.append(leaf_node)
         return children
 
+    def get_hierarchy_taxon_for_leaf(self, leaf: TreeLeafReference):
+        if not leaf.lineage:
+            return None
+        else:
+            return self.taxa_trie.get_taxon(leaf.lineage.split(self.taxa_trie.lin_sep)[-1])
+
     def get_leaf_nodes_for_taxon(self, taxon_name: str) -> list:
         """
         Find all of the leaf nodes (TreeLeafReference objects) in a reference package that represent a taxon
@@ -491,12 +543,15 @@ class ReferencePackage:
         Loads unique taxa found in the tree into a dictionary where the values are all leaves of a taxon.
 
         :param rank_name: Name of the rank to search for taxa
-        :return: A dictionary mapping unique taxa of rank 'rank_name' mapped to lists of
+        :return:
+          1. A dictionary mapping unique taxa of rank 'rank_name' mapped to lists of leaf node names
+           (format = leaf.number_self.prefix)
+          2. A dictionary mapping leaf.numbers to Taxon instances
         """
         # Check rank name to see if its present in the hierarchy
         if rank_name not in self.taxa_trie.accepted_ranks_depths:
-            self.taxa_trie.so_long_and_thanks_for_all_the_fish("Rank '{}' "
-                                                               "is not in hierarchy's accepted names.\n".format(rank_name))
+            self.taxa_trie.so_long_and_thanks_for_all_the_fish("Rank '{}' is not in hierarchy's accepted names.\n"
+                                                               "".format(rank_name))
 
         taxon_leaf_map = {}
         unique_taxa = {}
@@ -538,7 +593,7 @@ class ReferencePackage:
             s_query = lineage.split(lin_sep)
             x = 0
             while x < min([len(s_query), len(s_target)]):
-                if re.search("unclassified|environmental sample|root", s_query[x], re.IGNORECASE):
+                if re.search("unclassified|environmental sample", s_query[x], re.IGNORECASE):
                     n_unclassified += 1
                     break
                 if s_query[x] != s_target[x]:
@@ -698,6 +753,10 @@ class ReferencePackage:
 
         # fasta
         ref_fasta_dict = read_fasta_to_dict(self.f__msa)
+        if len(ref_fasta_dict) == 0:
+            logging.error("No sequences were read from Reference Package {}'s FASTA '{}'.\n"
+                          "".format(self.prefix, self.f__msa))
+            sys.exit(13)
         off_target_ref_headers = [ref_num + '_' + self.prefix for ref_num in self.lineage_ids]
         if len(off_target_ref_headers) == 0:
             logging.error("No reference sequences were retained for building testing " + target_clade + "\n")
@@ -731,7 +790,7 @@ class ReferencePackage:
                 fast_info.write(stdout + "\n")
             logging.info("done.\n")
         else:
-            ref_tree = Tree(self.f__tree)
+            ref_tree = self.get_ete_tree()
             ref_tree.prune(off_target_ref_headers, preserve_branch_length=True)
             logging.debug("\t" + str(len(ref_tree.get_leaves())) + " leaves in pruned tree.\n")
             ref_tree.write(outfile=self.f__tree, format=5)
@@ -824,12 +883,41 @@ class ReferencePackage:
         return
 
     def load_pfit_params(self, build_param_line):
+        """
+        Loads the polynomial classifier parameters that are stored in the ReferencePackage.pfit attribute
+        """
         build_param_fields = build_param_line.split("\t")
         if build_param_fields[8]:
             self.pfit = [float(x) for x in build_param_fields[8].split(',')]
         return
 
+    def enumerate_taxonomic_lineages(self) -> dict:
+        """
+        At times, it is handy to know how many of each taxonomic label are present in the reference package.
+        enumerate_taxonomic_lineages() iterates over the ranks (as the numeric representation) and sums the number of
+        taxa that are descendents of each broader taxonomic rank.
+        This leads to multiple-counts of each taxon by design.
+
+        :return: A dictionary mapping unique taxa in the ReferencePackage to their respective count
+        """
+        rank = 1
+        taxonomic_counts = dict()
+        while rank < 9:
+            for leaf in self.generate_tree_leaf_references_from_refpkg():
+                lineage = leaf.lineage.split(self.taxa_trie.lin_sep)
+                if len(lineage) < rank:
+                    continue
+                taxonomy = "; ".join(lineage[:rank])
+                if taxonomy not in taxonomic_counts:
+                    taxonomic_counts[taxonomy] = 0
+                taxonomic_counts[taxonomy] += 1
+            rank += 1
+        return taxonomic_counts
+
     def all_possible_assignments(self):
+        """
+        Returns a StringTrie (prefix-tree) representation of the taxonomic lineages of the ReferencePackage
+        """
         if len(self.lineage_ids) == 0:
             logging.error("ReferencePackage.lineage_ids is empty - information hasn't been slurped up yet.\n")
             sys.exit(17)
@@ -843,6 +931,50 @@ class ReferencePackage:
 
         return load_taxonomic_trie(lineage_list)
 
+    def taxonomically_label_tree(self) -> Tree:
+        """
+        When deciding what the taxonomic label should be assigned to a query sequence, algorithms may require a tree
+        (ete3.Tree instance) for which each node contains a 'taxon' instance, itself being a taxonomic_hierarchy.Taxon
+        instance.
+        This function assures that the phylogeny is strictly bifurcating, by using ete3's resolve_polytomy() function.
+        To label the nodes of a phylogeny,
+        """
+        # Generate an instance of the ETE3 Master Tree class
+        rt = self.get_ete_tree()
+
+        # Ensure that every taxonomic lineage is rooted by setting the parent of each 'domain' taxon to "r__Root"
+        root_taxon = self.taxa_trie.find_root_taxon()
+        self.taxa_trie.root_domains(root_taxon)
+
+        # Propagate a 'taxon' feature - None by default - to all TreeNodes for holding Taxon instances
+        for n in rt.traverse(strategy="postorder"):  # type: Tree
+            n.add_feature(pr_name="taxon", pr_value=None)
+
+        # Add the taxonomic lineages as features to each of the leaf nodes
+        for leaf_node in self.generate_tree_leaf_references_from_refpkg():  # type: TreeLeafReference
+            taxon = self.get_hierarchy_taxon_for_leaf(leaf_node)
+            tree_node = rt.get_leaves_by_name(name="{}_{}".format(leaf_node.number, self.prefix)).pop()
+            tree_node.add_feature(pr_name="taxon", pr_value=taxon)
+
+        # Label the internal nodes based on the LCA of the leaf nodes
+        for n in rt.traverse(strategy="postorder"):
+            if len(n.children) == 0:  # this is a leaf node
+                continue
+            elif len(n.children) == 1:
+                self.bail("Unexpected number of children (1) for Tree.Node {}.\n".format(n.name))
+                raise AssertionError
+            elif len(n.children) > 2:
+                n.resolve_polytomy(recursive=False)
+                for c in n.children:
+                    if not hasattr(c, "taxon"):
+                        c.add_feature(pr_name="taxon", pr_value=None)
+                        c.taxon = Taxon.lca(c.children[0].taxon, c.children[1].taxon)
+            l_node, r_node = n.get_children()
+            lca = Taxon.lca(l_node.taxon, r_node.taxon)
+            n.taxon = lca
+
+        return rt
+
 
 def view(refpkg: ReferencePackage, attributes: list) -> None:
     view_dict = {}
@@ -872,7 +1004,7 @@ def edit(refpkg: ReferencePackage, attributes: list, output_dir, overwrite: bool
         logging.error("`treesapp package edit` requires a value to change.\n")
         sys.exit(3)
     else:
-        k, v = attributes
+        k, v = attributes  # type: (str, str)
 
     try:
         current_v = refpkg.__dict__[k]
@@ -882,6 +1014,19 @@ def edit(refpkg: ReferencePackage, attributes: list, output_dir, overwrite: bool
 
     logging.info("Replacing attribute '{}' (currently '{}')\n".format(k, current_v))
 
+    if k.startswith("f__"):  # This is a file that is being modified and the contents need to be read
+        k_content = re.sub(r"^f__", '', k)
+        if not os.path.isfile(v):
+            logging.error("Unable to find path to new file '{}'. "
+                          "Exiting and the reference package '{}' will remain unchanged.\n".format(v, refpkg.f__json))
+            sys.exit(5)
+        with open(v) as content_handler:
+            v_content = content_handler.readlines()
+        try:
+            refpkg.__dict__[k_content] = v_content
+        except KeyError:
+            logging.error("Unable to find ReferencePackage attribute '{}' in '{}'.\n".format(k_content, refpkg.f__json))
+            sys.exit(7)
     refpkg.__dict__[k] = v
     if not overwrite:
         refpkg.f__json = os.path.join(output_dir, os.path.basename(refpkg.f__json))
