@@ -4,6 +4,7 @@ import logging
 import sys
 import re
 from pygtrie import StringTrie
+from ete3 import Tree
 
 from treesapp.phylo_seq import TreeLeafReference
 
@@ -27,6 +28,12 @@ class Taxon:
         self.coverage = 1
 
     def prefix_taxon(self, taxon_sep="__"):
+        if not self.prefix:
+            if not self.rank:
+                logging.error("Unable to return Taxon '{}' with its rank-prefix as its rank attribute is not set.\n"
+                              "".format(self.name))
+            else:
+                self.prefix = self.rank[0]
         return self.prefix + taxon_sep + self.name
 
     def lineage(self) -> list:
@@ -71,8 +78,11 @@ class Taxon:
 
     @staticmethod
     def lca(left_taxon, right_taxon):
-        l1 = left_taxon.lineage()
-        l2 = right_taxon.lineage()
+        try:
+            l1 = left_taxon.lineage()
+            l2 = right_taxon.lineage()
+        except AttributeError:
+            return
 
         while len(l1) > len(l2):
             l1.pop()
@@ -115,6 +125,7 @@ class TaxonomicHierarchy:
         self.no_rank_name = "no rank"  # Name of rank for non-canonical taxonomic ranks
         self.lin_sep = sep
         self.taxon_sep = "__"  # Separator between the rank prefix and the taxon name
+        self.root_taxon = "r" + self.taxon_sep + "Root"
         self.bad_taxa = ["cellular organisms", "unclassified"]  # Optional list that can be used to remove some taxa
         self.canonical_prefix = re.compile(r"^[nrdpcofgs]" + re.escape(self.taxon_sep))
         self.proper_species_re = re.compile("^(s__)?[A-Z][a-z]+ [a-z]+$")
@@ -126,6 +137,7 @@ class TaxonomicHierarchy:
         self.rank_prefix_map = {self.no_rank_name[0]: {self.no_rank_name},  # Tracks prefixes representing ranks
                                 'r': {"root"}}
         # The following are used for tracking the state of the instance's data structures
+        self.rooted = False
         self.trie_key_prefix = True  # Keeps track of the trie's prefix for automated updates
         self.trie_value_prefix = False  # Keeps track of the trie's prefix for automated updates
         self.clean_trie = True  # To track whether taxa of "no rank" are included in the trie
@@ -153,13 +165,32 @@ class TaxonomicHierarchy:
         logging.error(summary_string)
         raise RuntimeError(17)
 
-    def get_taxon_names(self, with_prefix=False):
+    def get_taxon_names(self, with_prefix=False) -> set:
+        """
+        Returns a set of all rank-prefixed taxa, from all ranks, in the TaxonomicHierarchy's hierarchy dictionary
+        """
         if with_prefix:
             return set([self.hierarchy[taxon].prefix_taxon() for taxon in self.hierarchy])
         else:
             return set([self.hierarchy[taxon].name for taxon in self.hierarchy])
 
-    def get_taxon(self, prefix_taxon) -> Taxon:
+    def get_rank_from_lineage(self, prefix_taxon: str, rank: str):
+        """
+        Retrieves the Taxon of a specific rank from the lineage of a Taxon instance.
+        If no ranks from the Taxon instances match the rank parameter then None is returned.
+
+        Example usage:
+            get_rank_from_lineage('s__Bos taurus', 'class')
+        """
+        taxon = self.get_taxon(prefix_taxon)
+        if not taxon:
+            return
+        for tax_name in taxon.lineage():  # type: Taxon
+            if tax_name.rank == rank:
+                return tax_name
+        return
+
+    def get_taxon(self, prefix_taxon: str) -> Taxon:
         """
         Retrieve the Taxon instance mapped to prefix_taxon in self.hierarchy.
         If prefix_taxon isn't in self.hierarchy, a warning is issued and None returned
@@ -174,6 +205,30 @@ class TaxonomicHierarchy:
             logging.debug("Taxon name '{}' not present in taxonomic hierarchy.\n".format(prefix_taxon))
             return
 
+    def get_bare_taxon(self, taxon_name: str):
+        """
+        A function for querying the hierarchy with a taxon that lacks a rank-prefix.
+
+        :param taxon_name: A string of a taxon that should be in TaxonomicHierarchy.hierarchy such as 'Bacteria'.
+        :return: Either a Taxon instance whose 'name' attribute matches taxon_name, or None
+        """
+        matches = []
+        if self.canonical_prefix.search(taxon_name):
+            self.so_long_and_thanks_for_all_the_fish("Taxon name ({}) has rank prefix. The function you want is "
+                                                     "TaxonomicHierarchy.get_taxon().\n".format(taxon_name))
+        for p_name, taxon in self.hierarchy.items():  # type: (str, Taxon)
+            if taxon.name == taxon_name:
+                matches.append(taxon)
+
+        if len(matches) == 1:
+            return matches.pop()
+        elif len(matches) > 1:
+            logging.warning("Multiple taxa in taxonomic hierarchy have the name '{}'.\n".format(taxon_name))
+            return
+        else:
+            logging.error("Unable to find taxon name '{}' in taxonomic hierarchy.\n".format(taxon_name))
+            sys.exit(3)
+
     def resolved_as(self, lineage, rank_name="species") -> bool:
         """
         This determines whether a lineage is resolved to at least the rank provided,
@@ -210,7 +265,72 @@ class TaxonomicHierarchy:
         except IndexError:
             self.so_long_and_thanks_for_all_the_fish("Empty taxon in lineage '{}'\n".format(lineage))
 
-    def rm_taxon_from_hierarchy(self, taxon: Taxon, decrement=1) -> None:
+    def find_root_taxon(self) -> Taxon:
+        """
+        Searches for the taxa that represent the root of the taxonomic hierarchy, returning the Taxon.prefix_taxon()
+        of which if found. If one is not found, a placeholder Taxon is added to self.hierarchy to fill the 'root' rank.
+
+        :return: A string of the rank-prefixed taxon, typically 'r__Root'
+        """
+        try:
+            root_reps = self.rank_representatives("root", with_prefix=True)
+            if len(root_reps) > 1:
+                self.so_long_and_thanks_for_all_the_fish("More than one taxon at rank of 'root' identified in"
+                                                         " taxonomic hierarchy.\n")
+                raise AssertionError
+            return self.get_taxon(root_reps.pop())
+        except KeyError:
+            logging.debug("No taxa at rank 'root' were present in reference package hierarchy - adding now.\n")
+            root_taxon = Taxon(name="Root", rank="root")
+            self.hierarchy[root_taxon.prefix_taxon()] = root_taxon
+            return root_taxon
+
+    def root_domains(self, root: Taxon, replace=True) -> Taxon:
+        """
+        Whether lineages are downloaded from a reference database or provided by the user, they tend to not include a
+        taxon that serves as the root, or LUCA, in a taxonomic hierarchy. However, this is essential for some of the
+        taxonomic hierarchy operations that TreeSAPP executes.
+
+        This function ensures that all domains in self.hierarchy have a parent that is the root Taxon instance.
+
+        :param root: A taxon that can be used
+        :param replace: Whether the Taxon currently occupying the root of the hierarchy should be replaced with the new Taxon
+        :return: The Taxon occupying the root rank in the hierarchy
+        """
+        if not isinstance(root, Taxon):
+            logging.error("Root taxon must be of type taxonomic_hierarchy.Taxon, not {}.\n".format(type(root)))
+            raise TypeError
+        if root.prefix_taxon() in self.hierarchy:
+            if replace:
+                self.redirect_hierarchy_paths(old=self.get_taxon(root.prefix_taxon()), rep=root)
+            else:
+                root = self.get_taxon(root.prefix_taxon())
+        else:
+            self.digest_taxon(taxon=root.name, previous=None, rank=root.rank, rank_prefix='r')
+        for domain_name in self.rank_representatives("domain", with_prefix=True):
+            domain_taxon = self.get_taxon(domain_name)
+            if domain_taxon.parent is None:
+                domain_taxon.parent = root
+        self.build_multifurcating_trie()
+        self.rooted = True
+        return root
+
+    def reroot_lineage(self, lineage: str) -> str:
+        """
+        Ensure the taxonomic lineage's most basal rank is Root (self.root_taxon)
+
+        :param lineage: A taxonomic lineage string
+        :return: A taxonomic lineage string with the
+        """
+        deep_rank = lineage.split(self.lin_sep)[0]
+        if not self.canonical_prefix.search(deep_rank):
+            self.so_long_and_thanks_for_all_the_fish("Prefixed taxon expected when re-rooting lineage.\n")
+            raise AssertionError
+        if deep_rank[0] != 'r':
+            lineage = self.root_taxon + self.lin_sep + lineage
+        return lineage
+
+    def scrub_taxon_from_hierarchy(self, taxon: Taxon, decrement=1) -> None:
         # Decrease Taxon.coverage for every Taxon instance in the lineage
         taxon.coverage -= decrement
         # Remove a Taxon from self.hierarchy dictionary if its coverage equals zero
@@ -223,20 +343,21 @@ class TaxonomicHierarchy:
                 # raise KeyError
         return
 
-    def redirect_hierarchy_paths(self, rep: Taxon, old: Taxon):
-        # Do not add values to rep since old is likely in its lineage so double counting
-        if rep not in old.lineage():  # Just to make sure though...
-            rep.absorb(old)
+    def redirect_hierarchy_paths(self, old: Taxon, rep=None):
         for taxon in self.hierarchy.values():  # type: Taxon
             if taxon.parent == old:
                 if taxon == rep:
                     pass
                 else:
                     taxon.parent = rep
-        # Remove all taxa between the old taxon and the LCA(rep, old)
-        lca = Taxon.lca(old, rep)
-        for taxon in Taxon.lineage_slice(old, lca):
-            self.rm_taxon_from_hierarchy(taxon)
+        if rep is not None:
+            # Do not add values to rep since old is likely in its lineage so double counting
+            if rep not in old.lineage():  # Just to make sure though...
+                rep.absorb(old)
+            # Remove all taxa between the old taxon and the LCA(rep, old)
+            lca = Taxon.lca(old, rep)
+            for taxon in Taxon.lineage_slice(old, lca):
+                self.scrub_taxon_from_hierarchy(taxon)
         return
 
     @staticmethod
@@ -249,6 +370,12 @@ class TaxonomicHierarchy:
 
     def resolve_conflicts(self) -> dict:
         """
+        Parses the tuples of Taxon instances (Taxon, Taxon) in self.conflicts and determines which instance is going to
+         be retained in the TaxonomicHierarchy.
+        The decision function is based on each Taxon's 'coverage' attribute and their taxonomic ranks.
+        If the ranks are the same, resolve_conflicts will retain the Taxon instance with greater coverage.
+        If one of the Taxon instance's rank is 'no_rank', the other will be used as the representative.
+        The last possibility is both have valid (and different) ranks, in which case both Taxon instances are retained.
 
         :return: A dictionary mapping the obsolete Taxon instance to the new representative Taxon instance
         """
@@ -616,7 +743,7 @@ class TaxonomicHierarchy:
         :param value_prefix: Flag indicating whether the taxon (node values) have their rank-prefix
         :return: None
         """
-        lineages = {"r__Root"}
+        lineages = {self.root_taxon}
 
         if key_prefix != self.trie_key_prefix:
             self.trie_key_prefix = key_prefix
@@ -661,6 +788,8 @@ class TaxonomicHierarchy:
 
     def get_prefixed_lineage_from_bare(self, bare_lineage: str) -> str:
         """
+        When given a lineage that lacks rank-prefixes, this function aims to fill them in.
+        NOTE: all taxa with no rank (e.g. cellular organisms, Terrabacteria group) will be removed.
 
         :param bare_lineage: A lineage lacking rank prefixes e.g. "Bacteria; Actinobacteria; Actinobacteria"
         :return: Empty string if lineage or a prefix of the full lineage isn't found, or a lineage with the correct
@@ -674,9 +803,24 @@ class TaxonomicHierarchy:
         lineage_split = bare_lineage.split(self.lin_sep)
         if self.clean_trie:
             lineage_split = self.rm_bad_taxa_from_lineage(lineage_split)  # Not guided by rank prefix
-            lineage_split = self.rm_absent_taxa_from_lineage(lineage_split)  # Not guided by rank prefix
+            self.rm_absent_taxa_from_lineage(lineage_split)  # Not guided by rank prefix
+        if not lineage_split:
+            logging.error("Unable to find a trace of the cleaned lineage '{}' (from {}) in the taxonomic hierarchy.\n"
+                          "".format(self.lin_sep.join(lineage_split), bare_lineage))
+            sys.exit(15)
 
         ref_lineage = ""
+        # While the first positions of the lineage are not in self.trie, prepend the root
+        if not self.query_trie(lineage_split[0]):
+            for prefix in self.trie:
+                if self.query_trie(prefix + self.lin_sep + lineage_split[0]):
+                    lineage_split = prefix.split(self.lin_sep) + lineage_split
+                    break
+        if not self.query_trie(lineage_split[0]):
+            logging.error("Unable to root the lineage '{}' (cleaned is {}) in the taxonomic hierarchy.\n"
+                          "".format(bare_lineage, self.lin_sep.join(lineage_split)))
+            sys.exit(17)
+
         # Iteratively climb the taxonomic lineage until a hit is found or there are no further ranks
         while not ref_lineage and lineage_split:
             taxon = self.query_trie(self.lin_sep.join(lineage_split))
@@ -738,22 +882,69 @@ class TaxonomicHierarchy:
             cleaned_lineage = split_lineage
         return cleaned_lineage
 
-    def rm_absent_taxa_from_lineage(self, lineage_list: list) -> list:
+    def rm_absent_taxa_from_lineage(self, lineage_list: list, with_prefix=False) -> None:
         """
         Removes all taxa from the taxonomic lineage that are not present in self.hierarchy.
         This is meant to remove potentially non-canonical ranks from the unprefixed taxonomic lineage.
 
         :param lineage_list: A list representation of the split taxonomic lineage
-        :return: A list of the taxonomic lineage with the taxa that are not present in self.hierarchy removed
+        :param with_prefix: Boolean controlling whether the inputs have a rank_prefix (True) or not (False)
+        :return: None
         """
-        _taxa = self.get_taxon_names()
+        _taxa = self.get_taxon_names(with_prefix=with_prefix)
         i = 0
         while i < len(lineage_list):
             if lineage_list[i] not in _taxa:
                 lineage_list.pop(i)
             else:
                 i += 1
-        return lineage_list
+        return
+
+    def prune_branches(self, tree, leaf_taxa_map: dict, rank="Genus"):
+        """
+        Function for removing leaves of unclassified and polyphyletic lineages
+
+        :type tree: Tree()
+        :param tree: An Environment for Tree Exploration (ETE) Tree object
+        :param leaf_taxa_map: A dictionary mapping tree leaf number keys to NCBI lineage strings
+        :param rank: A taxonomic rank to test for monophyly
+        :return: pruned_nodes dict() of Tree() nodes
+        """
+        pruned_nodes = dict()
+        if not isinstance(tree, Tree):
+            logging.error("Tree is not ete tree object.\n")
+            raise AssertionError()
+        # Check to see if the two collections are comparable
+        for leaf in tree:
+            if leaf.name not in leaf_taxa_map.keys():
+                logging.error(str(leaf.name) + " not found in leaf_taxa_map.\n")
+                raise AssertionError("Leaves in tree and tax_ids file are disparate sets.\n")
+        # Raw lineages are too specific to test for monophyly, so try at a deeper rank by trimming the lineages
+        leaf_taxa_map = self.trim_lineages_to_rank(leaf_taxa_map, rank)
+
+        # Add the lineages to the Tree instance
+        for leaf in tree:
+            leaf.add_features(lineage=leaf_taxa_map.get(leaf.name, "none"))
+
+        # Print the tree for debugging:
+        # print(tree.get_ascii(attributes=["name", "lineage"], show_internal=False))
+
+        # Add all the monophyletic leaf node numbers to pruned_nodes
+        unique_lineages = sorted(list(set(leaf_taxa_map.values())))
+        for lineage in unique_lineages:
+            pruned_nodes[lineage] = dict()
+            acc = 1  # In case lineages are scattered (e.g. paralogs in the tree) these need to be indexed
+            for node in tree.get_monophyletic(values=[lineage], target_attr="lineage"):
+                leaf_descendents = node.get_leaves()
+                if len(leaf_descendents) == 1:
+                    pass
+                elif len(leaf_descendents) > 1:
+                    # This is an internal node
+                    pruned_nodes[lineage][acc] = leaf_descendents
+                    acc += 1
+                else:
+                    raise AssertionError("Expected at least one leaf leading from node " + str(node.name))
+        return pruned_nodes
 
     def strip_rank_prefix(self, lineage: str) -> str:
         stripped_lineage = []
@@ -776,39 +967,73 @@ class TaxonomicHierarchy:
         """
         reconstructed_lineage = []
         self.validate_rank_prefixes()
-        for rank in lineage.split(self.lin_sep):  # type: str
+        for taxon in lineage.split(self.lin_sep):  # type: str
             try:
-                prefix, name = rank.split(self.taxon_sep)
+                prefix, name = taxon.split(self.taxon_sep)
             except ValueError:
-                rank = re.sub(r'(?<!^[a-z])' + re.escape(self.taxon_sep), '_', rank)
+                taxon = re.sub(r'(?<!^[a-z])' + re.escape(self.taxon_sep), '_', taxon)
                 try:
-                    prefix, name = rank.split(self.taxon_sep)
+                    prefix, name = taxon.split(self.taxon_sep)
                 except ValueError:
                     self.so_long_and_thanks_for_all_the_fish("Rank-prefix required for clean_lineage_string().\n"
                                                              "None was provided for taxon '{}' in lineage {}\n"
-                                                             "".format(rank, lineage))
+                                                             "".format(taxon, lineage))
                     raise ValueError()
 
-            if not self.no_rank_re.match(rank) and self.rank_prefix_map[prefix] in self.accepted_ranks_depths:
-                if with_prefix is False:
-                    rank = self.canonical_prefix.sub('', rank)
-                if name:
-                    reconstructed_lineage.append(str(rank))
+            try:
+                if not self.no_rank_re.match(taxon) and self.rank_prefix_map[prefix] in self.accepted_ranks_depths:
+                    if with_prefix is False:
+                        taxon = self.canonical_prefix.sub('', taxon)
+                    if name:
+                        reconstructed_lineage.append(str(taxon))
+            except KeyError:
+                raise KeyError
         return self.lin_sep.join(reconstructed_lineage)
 
+    def match_organism(self, organism: str, lineage: list) -> str:
+        """
+        Searches the taxonomic hierarchy's prefix tree (StringTrie instance) for organisms that are descendents
+        of the provided lineage.
+
+        :param organism: The organism name to query against the prefix tree
+        :param lineage: A list of taxa that are part of a single lineage. These are to be joined,
+        using the TaxonomicHierarchy's lin_sep attribute, to create the prefix tree query.
+        :return: Name of the organism with a rank-prefix if the organism was found to be a descendent of lineage.
+        """
+        if isinstance(lineage, str):
+            logging.error("Lineage was string when list was expected.\n")
+            raise TypeError
+
+        if len(self.trie) == 0 or self.trie_key_prefix is False:
+            self.build_multifurcating_trie(key_prefix=True, value_prefix=False)
+
+        if self.canonical_prefix.search(organism):
+            return organism
+        else:
+            if self.lin_sep.join(lineage) not in self.trie:
+                logging.error("Lineage elements are not present in prefix tree: '{}'.\n".format("; ".join(lineage)))
+                sys.exit(13)
+
+            descendents = self.trie.items(prefix=self.lin_sep.join(lineage))
+            for child_lineage, taxon in descendents:  # type: (str, str)
+                if taxon == organism:
+                    return child_lineage.split(self.lin_sep)[-1]
+            return ""
+
     def check_lineage(self, lineage: str, organism: str, verbosity=0) -> str:
         """
         Sometimes the NCBI lineage is incomplete or the rank prefixes are out of order.
         This function checks (and fixes) the following things:
         1. Uses organism to add Species to the lineage if it is missing
-        2. Ensure the progression of rank (i.e. from domain to species) is ordered properly
+        2. Adds the root taxon to the taxonomic lineage if it is missing and present in self.hierarchy
+        3. Ensure the progression of rank (i.e. from root to species) is ordered properly
 
         :param lineage: A taxonomic lineage *with prefixed ranks* separated by self.lin_sep
         :param organism: Name of the organism. Parsed from the sequence header (usually at the end in square brackets)
         :param verbosity: 1 prints debugging messages
         :return: A list of elements for each taxonomic rank representing the taxonomic lineage
         """
-        if not self.trie_key_prefix or not self.clean_trie:
+        if not self.trie_key_prefix or not self.clean_trie or len(self.trie) == 0:
             self.clean_trie = True
             logging.debug("Switching multifurcating trie to include rank prefixes.\n")
             self.build_multifurcating_trie(key_prefix=True)
@@ -821,38 +1046,38 @@ class TaxonomicHierarchy:
                           "clean_trie = {3}\n".format(lineage, organism, self.trie_key_prefix, self.clean_trie))
 
         lineage = self.clean_lineage_string(lineage, with_prefix=True)
+        lineage_list = lineage.split(self.lin_sep)
+        self.rm_absent_taxa_from_lineage(lineage_list, with_prefix=True)
 
-        if len(lineage) == 0:
-            return ""
+        if len(lineage_list) == 0:
+            self.so_long_and_thanks_for_all_the_fish("Taxon '{}' from lineage '{}'"
+                                                     " is not in TaxonomicHierarchy.\n".format(organism, lineage))
+            raise RuntimeError
 
-        if not self.project_lineage(lineage):
-            self.so_long_and_thanks_for_all_the_fish("Lineage '{0}' not in taxonomic hierarchy.\n".format(lineage))
+        hierarchy_taxon = self.get_taxon(prefix_taxon=lineage_list[-1])
+
+        # Ensure the lineage is rooted, and if not, ensure all domains are properly rooted
+        taxon_lineage = [t for t in hierarchy_taxon.lineage() if t.rank in self.accepted_ranks_depths]
+        if taxon_lineage[0].rank != "root":
+            self.root_domains(self.find_root_taxon())
+            taxon_lineage = [t for t in hierarchy_taxon.lineage() if t.rank in self.accepted_ranks_depths]
+
+        lineage_list = [t.prefix_taxon() for t in taxon_lineage]
 
         # Handle prefix discrepancy between lineage and organism if organism doesn't have rank prefix
-        if not self.canonical_prefix.search(organism):
-            for child_lineage, taxon in self.trie.items(prefix=lineage):  # type: (str, str)
-                if taxon == organism:
-                    organism = child_lineage.split(self.lin_sep)[-1]
-
-        lineage_list = lineage.split(self.lin_sep)
-        rank_resolution = self.resolved_to(lineage)
-        try:
-            rank_depth = self.accepted_ranks_depths[rank_resolution]
-        except KeyError:
-            logging.error("'{}' not in list of accepted ranks ({}).\n"
-                          "".format(rank_resolution, ", ".join(self.accepted_ranks_depths.keys())))
-            raise KeyError
-        if self.proper_species_re.match(lineage_list[-1]):
+        organism = self.match_organism(organism, lineage_list)
+        # Determine the state of completeness for the taxon's lineage
+        if hierarchy_taxon.rank == "species" or self.proper_species_re.match(lineage_list[-1]):
             if verbosity:
                 logging.debug("check_lineage(): Perfect lineage.\n")
-        elif len(lineage_list) == 6 and rank_depth == 6 and self.proper_species_re.match(organism):
+        elif hierarchy_taxon.rank == "genus" and self.proper_species_re.match(organism):
             if not self.canonical_prefix.search(organism):
                 if self.rank_prefix_map['s'] == "species":
                     organism = "s" + self.taxon_sep + organism
                 else:
                     self.so_long_and_thanks_for_all_the_fish("Unexpected rank prefix for species"
                                                              " in TaxonomicHierarchy.rank_prefix_map\n")
-            self.append_to_hierarchy_dict(organism, lineage_list[-1], rank="species", rank_prefix='s')
+            self.append_to_hierarchy_dict(child=organism, parent=lineage_list[-1], rank="species", rank_prefix='s')
             lineage_list.append(organism)
             if verbosity:
                 logging.debug("check_lineage(): Organism name added to complete the lineage.\n")
@@ -861,38 +1086,31 @@ class TaxonomicHierarchy:
                 logging.debug("check_lineage(): Truncated lineage.\n")
 
         self.validate_rank_prefixes()  # Ensure the rank-prefix map is formatted correctly
-        # Ensure the order and progression of ranks is correct (no domain -> phylum -> species for example)
+        # Ensure the order and progression of ranks is correct (domain -> phylum -> species for example)
         i = 0
-        for taxon in lineage_list:  # type: str
-            try:
-                prefix, _ = taxon.split(self.taxon_sep)
-            except ValueError:
-                self.so_long_and_thanks_for_all_the_fish("Rank-prefix required for check_lineage(),"
-                                                         " taxon: '{}'.\n".format(taxon))
-                raise ValueError
-
-            if self.rank_prefix_map[prefix] not in self.accepted_ranks_depths:
+        for taxon in taxon_lineage:  # type: Taxon
+            if self.rank_prefix_map[taxon.prefix] not in self.accepted_ranks_depths:
                 logging.debug("Rank '{0}' is not in the list of accepted taxonomic ranks.\n"
-                              "Lineage will be truncated to '{1}'.\n".format(self.rank_prefix_map[prefix],
+                              "Lineage will be truncated to '{1}'.\n".format(self.rank_prefix_map[taxon.prefix],
                                                                              self.lin_sep.join(lineage_list[0:i])))
                 for prefix_taxon_name in lineage_list[i:]:
                     if self.get_taxon(prefix_taxon=prefix_taxon_name):
-                        self.rm_taxon_from_hierarchy(self.get_taxon(prefix_taxon=prefix_taxon_name))  # type: Taxon
+                        self.scrub_taxon_from_hierarchy(self.get_taxon(prefix_taxon=prefix_taxon_name))  # type: Taxon
                 lineage_list = lineage_list[0:i]
                 break
 
-            if self.accepted_ranks_depths[self.rank_prefix_map[taxon[0]]] > i+1:
+            if self.accepted_ranks_depths[self.rank_prefix_map[taxon.prefix]] > i:
                 logging.debug("Order of taxonomic ranks in cleaned lineage '{0}' is unexpected.\n"
                               "Lineage will be truncated to '{1}'.\n".format(self.lin_sep.join(lineage_list),
                                                                              self.lin_sep.join(lineage_list[0:i])))
                 for prefix_taxon_name in lineage_list[i:]:
                     if self.get_taxon(prefix_taxon=prefix_taxon_name):
-                        self.rm_taxon_from_hierarchy(self.get_taxon(prefix_taxon=prefix_taxon_name))  # type: Taxon
+                        self.scrub_taxon_from_hierarchy(self.get_taxon(prefix_taxon=prefix_taxon_name))  # type: Taxon
                 lineage_list = lineage_list[0:i]
                 break
             i += 1
         if len(lineage_list) == 0:
-            lineage_list = ["r__Root"]
+            lineage_list = [self.root_taxon]
 
         return self.lin_sep.join(lineage_list)
 
@@ -908,11 +1126,11 @@ class TaxonomicHierarchy:
         """
         if type(taxa) is str:
             taxa = [taxa]
-        for prefix_taxon_name in sorted(taxa):
+        for prefix_taxon_name in sorted(taxa):  # type: str
             leaf_taxon = self.get_taxon(prefix_taxon=prefix_taxon_name)  # type: Taxon
             if leaf_taxon:
                 for taxon in leaf_taxon.lineage():
-                    self.rm_taxon_from_hierarchy(taxon)
+                    self.scrub_taxon_from_hierarchy(taxon)
                 # Update self.lineages_fed
                 self.lineages_fed -= 1
 
@@ -1003,26 +1221,57 @@ class TaxonomicHierarchy:
 
         self.validate_rank_prefixes()
 
-        for node_name in sorted(leaf_taxa_map):
+        for node_name in sorted(leaf_taxa_map):  # type: str
             lineage = leaf_taxa_map[node_name].split(self.lin_sep)
+            base_taxon = self.get_taxon(lineage[0])
+            leaf_taxon = self.get_taxon(lineage[-1])
 
-            # Remove lineage from testing if the rank doesn't exist (unclassified at a high rank)
-            if len(lineage) == 1 or len(lineage) < depth:
-                truncated += 1
+            if not base_taxon or not leaf_taxon:
+                logging.debug("Unable to trim lineage '{}' as some taxa were not found in taxonomic hierarchy.\n"
+                              "".format(leaf_taxa_map[node_name]))
                 continue
 
-            trimmed_lineage_map[node_name] = self.lin_sep.join(lineage[:depth])
+            post = True
+            lin = []
+            for taxon in reversed(leaf_taxon.lineage()):  # type: Taxon
+                if taxon.prefix_taxon() not in lineage:
+                    continue
+                if taxon.rank == rank:
+                    post = False
+                if not post:
+                    lin.append(taxon.prefix_taxon())
+                else:
+                    continue
+            if lin:
+                trimmed_lineage_map[node_name] = self.lin_sep.join(reversed(lin))
 
-            # Ensure the last taxon in the lineage has the expected rank-prefix
-            taxon = lineage[:depth][-1]
-            try:
-                th_rank_name = self.rank_prefix_map[taxon[0]]
-            except KeyError:
-                self.so_long_and_thanks_for_all_the_fish("Rank prefix {}"
-                                                         " not found in rank prefix map.\n".format(taxon[0]))
-            if rank != th_rank_name:
-                self.so_long_and_thanks_for_all_the_fish("Rank prefix '{}' doesn't match rank name '{}'"
-                                                         " in trimmed lineage.\n".format(rank, th_rank_name))
+            # if base_taxon.prefix_taxon() == self.root_taxon:
+            #     modifier = 1
+            # elif base_taxon.rank != "domain":
+            #     logging.error("Lineage '{}' doesn't begin with a taxon representing either root or domain rank.\n")
+            #     sys.exit(17)
+            # else:
+            #     modifier = 0
+            #
+            # # Remove lineage from testing if the rank doesn't exist (unclassified at a high rank)
+            # if len(lineage) == 1 or len(lineage) < (depth+modifier):
+            #     truncated += 1
+            #     continue
+            #
+            # trimmed_lineage_map[node_name] = lineage[:(depth+modifier)]
+            #
+            # # Ensure the last taxon in the lineage has the expected rank-prefix
+            # taxon = trimmed_lineage_map[node_name][-1]
+            # try:
+            #     th_rank_name = self.rank_prefix_map[taxon[0]]
+            # except KeyError:
+            #     self.so_long_and_thanks_for_all_the_fish("Rank prefix {}"
+            #                                              " not found in rank prefix map.\n".format(taxon[0]))
+            # if rank != th_rank_name:
+            #     self.so_long_and_thanks_for_all_the_fish("Rank prefix '{}' doesn't match rank name '{}'"
+            #                                              " in trimmed lineage.\n".format(rank, th_rank_name))
+            #
+            # trimmed_lineage_map[node_name] = self.lin_sep.join(trimmed_lineage_map[node_name])
 
         logging.debug("{0} lineages truncated before '{1}' were removed during lineage trimming.\n".format(truncated,
                                                                                                            rank))
