@@ -6,7 +6,6 @@ import time
 import itertools
 from glob import glob
 
-from ete3 import Tree
 from tqdm import tqdm
 import numpy as np
 import seaborn
@@ -18,11 +17,10 @@ from sklearn import model_selection, svm, metrics, preprocessing, manifold
 from treesapp import file_parsers
 from treesapp import wrapper
 from treesapp import fasta
-from treesapp.phylo_seq import PQuery
+from treesapp.phylo_seq import PQuery, PhyloPlace
 from treesapp.external_command_interface import launch_write_command, create_dir_from_taxon_name
-from treesapp.jplace_utils import jplace_parser
+from treesapp.jplace_utils import jplace_parser, demultiplex_pqueries, calc_pquery_mean_tip_distances
 from treesapp.entish import map_internal_nodes_leaves
-from treesapp.phylo_dist import parent_to_tip_distances
 from treesapp.refpkg import ReferencePackage
 
 
@@ -69,7 +67,7 @@ def summarize_query_classes(positives: set, query_seq_names: set) -> None:
     return
 
 
-def bin_headers(assignments: dict, annot_map: dict, entrez_query_dict: dict) -> (dict, dict, dict):
+def bin_headers(assignments: dict, annot_map: dict, entrez_query_dict: dict, refpkgs: dict) -> (dict, dict, dict):
     """
     Function for sorting/binning the classified sequences at T/F positives/negatives based on the annot_map file
     that specifies which queries belong to which orthologous group/protein family and reference package.
@@ -77,6 +75,7 @@ def bin_headers(assignments: dict, annot_map: dict, entrez_query_dict: dict) ->
     :param assignments: Dictionary mapping the ReferencePackage.prefix to TreeProtein instances
     :param annot_map: Dictionary mapping reference package (gene) name keys to database names values
     :param entrez_query_dict: Dictionary mapping sequence names to their respective EntrezRecord instances
+    :param refpkgs: A dictionary mapping reference package prefixes to its ReferencePackage instance
     :return: None
     """
     # False positives: those that do not belong to the annotation matching a reference package name
@@ -101,9 +100,13 @@ def bin_headers(assignments: dict, annot_map: dict, entrez_query_dict: dict) ->
         try:
             positives = positive_queries[refpkg_name]
         except KeyError:
-            logging.error("Unable to find '{}' in the set of positive queries:\n".format(refpkg_name) +
-                          ", ".join([str(n) for n in positive_queries.keys()]) + "\n")
-            sys.exit(5)
+            refpkg = refpkgs[refpkg_name]  # type: ReferencePackage
+            if refpkg.refpkg_code in positive_queries:
+                positives = positive_queries[refpkg.refpkg_code]
+            else:
+                logging.error("Unable to find '{}' in the set of positive queries:\n".format(refpkg_name) +
+                              ", ".join([str(n) for n in positive_queries.keys()]) + "\n")
+                sys.exit(5)
         true_positives = set()
         tp[refpkg_name] = list()
         fp[refpkg_name] = set()
@@ -213,7 +216,7 @@ def generate_pquery_data_for_trainer(ref_pkg: ReferencePackage, taxon: str,
 
     ce_fasta = fasta.FASTA(ce_refpkg.f__msa)
     ce_fasta.load_fasta()
-    ce_tree = Tree(ce_refpkg.f__tree)
+    ce_tree = ce_refpkg.taxonomically_label_tree()
 
     # Paths to temporary files
     query_fasta_file = taxon_test_dir + "queries.fa"
@@ -277,36 +280,16 @@ def generate_pquery_data_for_trainer(ref_pkg: ReferencePackage, taxon: str,
     jplace_data = jplace_parser(raxml_files["jplace"])
     placement_tree = jplace_data.tree
     node_map = map_internal_nodes_leaves(placement_tree)
-    for pquery in jplace_data.placements:
-        top_lwr = 0.1
-        top_placement = PQuery(taxon, rank)
-        top_placement.ref_name = ref_pkg.prefix
-        for name, info in pquery.items():
-            if name == 'p':
-                for placement in info:
-                    # Only record the best placement's distance
-                    lwr = float(placement[2])
-                    if lwr > top_lwr:
-                        top_lwr = lwr
-                        top_placement.inode = placement[0]
-                        top_placement.likelihood = placement[1]
-                        top_placement.lwr = lwr
-                        top_placement.distal = round(float(placement[3]), 6)
-                        top_placement.pendant = round(float(placement[4]), 6)
-                        leaf_children = node_map[int(top_placement.inode)]
-                        if len(leaf_children) > 1:
-                            # Reference tree with clade excluded
-                            parent = ce_tree.get_common_ancestor(leaf_children)
-                            tip_distances = parent_to_tip_distances(parent, leaf_children)
-                            top_placement.mean_tip = round(float(sum(tip_distances) / len(tip_distances)), 6)
-            elif name == 'n':
-                top_placement.place_name = query_seq_name_map[int(info.pop())]
-            else:
-                logging.error("Unexpected variable in pquery keys: '" + name + "'\n")
-                sys.exit(33)
+    jplace_data.pqueries = demultiplex_pqueries(jplace_data=jplace_data)
+    calc_pquery_mean_tip_distances(jplace_data, internal_node_leaf_map=node_map)
+    for pquery in jplace_data.pqueries:  # type: PQuery
+        pquery.ref_name = ref_pkg.prefix
+        pquery.rank = rank
+        pquery.lineage = taxon
+        pquery.process_max_weight_placement(ce_tree)
 
-        if top_placement.lwr >= 0.5:  # The minimum likelihood weight ration a placement requires to be included
-            pqueries.append(top_placement)
+        if pquery.consensus_placement.like_weight_ratio >= 0.5:
+            pqueries.append(pquery)
 
     # Remove intermediate files from the analysis of this taxon
     intermediate_files += list(raxml_files.values())
@@ -367,6 +350,7 @@ a numpy array from each query's data:
         internal_nodes = refpkg.get_internal_node_leaf_map()
 
         for pquery in pqueries:  # type: PQuery
+            placement = pquery.consensus_placement  # type: PhyloPlace
             if annot_map:
                 ref_name = annot_map[pquery.ref_name]
             else:
@@ -374,19 +358,20 @@ a numpy array from each query's data:
             if pquery.place_name in condition_names[ref_name]:
                 # Calculate the features
                 try:
-                    distal, pendant, avg = pquery.distal, pquery.pendant, pquery.mean_tip
+                    distal, pendant, avg = placement.distal_length, placement.pendant_length, placement.mean_tip_length
                 except AttributeError:
                     distal, pendant, avg = [round(float(x), 3) for x in pquery.distances.split(',')]
 
-                lwr_bin = round(float(pquery.lwr), 2)
+                lwr_bin = round(float(placement.like_weight_ratio), 2)
                 # hmm_perc = round((int(pquery.seq_len)*100)/refpkg.profile_length, 1)
 
                 try:
-                    leaf_children = len(internal_nodes[int(pquery.inode)])
+                    leaf_children = len(internal_nodes[int(placement.edge_num)])
                 except KeyError:
                     logging.error("Unable to find internal node '{}' in the {} node-leaf map indicating a discrepancy "
                                   "between reference package versions used by treesapp assign and those used here.\n"
-                                  "Was the correct output directory provided?".format(pquery.inode, pquery.ref_name))
+                                  "Was the correct output directory provided?"
+                                  "".format(placement.edge_num, pquery.ref_name))
                     sys.exit(5)
 
                 raw_array = np.array([leaf_children, pquery.evalue, lwr_bin, distal, pendant, avg])
